<?php
/**
 * @copyright	Copyright (C) 2015 Spiral Scripts. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */

// no direct access
defined('_JEXEC') or die;


abstract class AmazonS3Helper
{
	static $keys =  array();
	
	
	
	public static function getParams($id = 1)
	{		
		
		$params = JComponentHelper::getParams('com_amazons3');
		$keys = self::getKeys($id);
		if($keys)
		{
			$params->set("secret_key",$keys->secretkey);
			$params->set("access_key",$keys->publickey);
			$params->set("key_data",$keys->data);
			$params->set("key_encrypted_data",$keys->encrypted_data);				
			$params->set("key_type",$keys->keytype);
		}
		return $params;

		
	}
	
	
	public static function blankKeys()
	{
			$blankrow = new stdClass();
			$blankrow->secretkey = "";
			$blankrow->publickey = "";
			$blankrow->data = "";
			$blankrow->encrypted_data = "";
            return $blankrow;		
		
	}
	
    public static function getKeys($id = 1)
	{
		
		if(isset(self::$keys[$id]))
		{
			return self::$keys[$id];
		}
		
		$word = self::getSecretWord();		
		$params = JComponentHelper::getParams('com_amazons3');
		$hash = $params->get("hash_type","SHA2");
		$validHashValues = array('SHA1','SHA2','MD5');
		if(!in_array($hash, $validHashValues))
		{
			$hash = "SHA2";
		}
		
		
		try{
			$db = JFactory::getDBO();
			$query = $db->getQuery(true);
			if($hash == "SHA2")
			{		  
			
			  $query->select(array("keytype","publickey","data","AES_DECRYPT( secretkey, UNHEX(".$hash."(".$db->quote($word).",512))) AS skey", "AES_DECRYPT( encrypted_data, UNHEX(".$hash."(".$db->quote($word).",512))) AS edata"));
			}
			else
			{
			  $query->select(array("keytype","publickey","data","AES_DECRYPT( secretkey, UNHEX(".$hash."(".$db->quote($word)."))) AS skey", "AES_DECRYPT( encrypted_data, UNHEX(".$hash."(".$db->quote($word)."))) AS edata"));				
			}
			$query->from("#__amazons3_config");
			//$query->where("id=".(int)$id." AND NOT ISNULL(secretkey)");
			$query->where("id=".(int)$id);
			
			$db->setQuery( $query );
			$row = $db->loadObject();
					
			if($row)
			{
				$row->secretkey = base64_decode($row->skey);
				$row->encrypted_data = base64_decode($row->edata);								
				self::$keys[$id] = $row;				
				return $row;
			}
			else
			{
				self::$keys[$id] = false;
				return false;	
			}
		} catch(Exception $e){ return false; }
		
	}
	
	/* legacy version just so old download plugin doesn't fail*/
	
    public static function getKey($id = 1)
	{
		
		$word = self::getSecretWord();		
		$params = JComponentHelper::getParams('com_amazons3');
		$hash = $params->get("hash_type","SHA2");
		
		try{
			$db = JFactory::getDBO();
			$query = $db->getQuery(true);
			if($hash == "SHA2")
			{
			  $query->select("AES_DECRYPT( secretkey, UNHEX(".$hash."('".$word."',512))) AS skey");
			}
			else
			{
			  $query->select("AES_DECRYPT( secretkey, UNHEX(".$hash."('".$word."'))) AS skey");				
			}
			$query->from("#__amazons3_config");
			$query->where("id=".(int)$id." AND NOT ISNULL(secretkey)");
			$db->setQuery( $query );
			$row = $db->loadResult();
					
			if($row)
			{
				$secretkey = base64_decode($row); //note that it is base64 encoded to prevent problems with binary data being saved incorrectly
				return $secretkey;
			}
			else
			{
				return false;	
			}
		} catch(Exception $e){ return false; }
		
	}

	
	/*
	 * Submenu for 
	 */
	public static function addSubmenu($vName = 'amazons3')
	{
		$input = JFactory::getApplication()->input;
		$option = $input->get("option","",'string');
		if($option == "com_categories")
		{
			self::addSidebar("amazons3");
			return;
		}
		
		$params = JComponentHelper::getParams("com_amazons3");
		$availableHosts = $params->get("availableHosts",array("all"));
		

		JSubMenuHelper::addEntry(JText::_('AS3_ADMIN'), 'index.php?option=com_amazons3&view=amazons3');
		if(in_array("all", $availableHosts) || in_array("amazons3", $availableHosts))
		{
			JSubMenuHelper::addEntry(JText::_('AS3_VIEWBUCKETS'), 'index.php?option=com_amazons3&view=buckets');
		}
		if(in_array("all", $availableHosts) || in_array("dropbox", $availableHosts))
		{		
			JSubMenuHelper::addEntry(JText::_('AS3_VIEWDROPBOX'), 'index.php?option=com_amazons3&view=dropbox');
		}
		JSubMenuHelper::addEntry(JText::_('AS3_VIEWPACKAGES'), 'index.php?option=com_amazons3&view=packages');
		JSubMenuHelper::addEntry(JText::_('AS3_VIEWFILES'), 'index.php?option=com_amazons3&view=files');
		JSubMenuHelper::addEntry(JText::_('JCATEGORIES'), 'index.php?option=com_categories&extension=com_amazons3');
		
						
	}
	
	public static function addSidebar($vName = 'amazons3')
	{

		$params = JComponentHelper::getParams("com_amazons3");
		$availableHosts = $params->get("availableHosts",array("all"));


		JHtmlSidebar::addEntry(JText::_('AS3_ADMIN'), 'index.php?option=com_amazons3&view=amazons3');
		if(in_array("all", $availableHosts) || in_array("amazons3", $availableHosts))
		{		
			JHtmlSidebar::addEntry(JText::_('AS3_VIEWBUCKETS'), 'index.php?option=com_amazons3&view=buckets');
		}
		if(in_array("all", $availableHosts) || in_array("dropbox", $availableHosts))
		{				
			JHtmlSidebar::addEntry(JText::_('AS3_VIEWDROPBOX'), 'index.php?option=com_amazons3&view=dropbox');	
		}
		JHtmlSidebar::addEntry(JText::_('AS3_VIEWPACKAGES'), 'index.php?option=com_amazons3&view=packages');
		JHtmlSidebar::addEntry(JText::_('AS3_VIEWFILES'), 'index.php?option=com_amazons3&view=files');	
		JHtmlSidebar::addEntry(JText::_('JCATEGORIES'), 'index.php?option=com_categories&extension=com_amazons3');

						
	}
	
	
	/**
	 * Gets a list of the actions that can be performed.
	 *
	 * @param   integer  The category ID.
	 *
	 * @return  JObject
	 * @since   1.6
	 */
	public static function getActions($categoryId = 0)
	{
		$user	= JFactory::getUser();
		$result	= new JObject;
	
		if (empty($categoryId))
		{
			$assetName = 'com_amazons3';
			$level = 'component';
		}
		else
		{
			$assetName = 'com_amazons3.category.'.(int) $categoryId;
			$level = 'category';
		}
	
		$actions = JAccess::getActions('com_amazons3', $level);
	
		foreach ($actions as $action)
		{
			$result->set($action->name,	$user->authorise($action->name, $assetName));
		}
	
		return $result;
	}	
	
	public static function getSecretWord()
	{
		if(class_exists("Amazons3Config"))
		{
			return Amazons3Config::$secret_word;
		}
		$filename = JPATH_ADMINISTRATOR."/components/com_amazons3/config.php";
		if(file_exists($filename))
		{
			require_once($filename);
			return Amazons3Config::$secret_word;
			
		}
		
		//secret word has not been generated yet
		
		$bytes = JCrypt::genRandomBytes(16);
		$hex   = bin2hex($bytes);
		

		$filename = JPATH_ADMINISTRATOR."/components/com_amazons3/config.php-dist";
		if(file_exists($filename))
		{
	        $filecontents = file_get_contents($filename);
			$filecontents = str_replace("xxxxxxxx",$hex,$filecontents);
			file_put_contents($filename, $filecontents);
			$newfilename = JPATH_ADMINISTRATOR."/components/com_amazons3/config.php";
			
			rename($filename, $newfilename);
			return $hex;
		}
		else
		{
			//something wrong here
			JError::raiseError( 4711, 'There is a problem with generating a secret word necessary for encrypting your secret key' );
		}

	}
	
	public static function filterBucketName($name)
	{
		//echo ' bucket name'.$name;
		
		if(strlen($name) < 3)
		{
			return "";
		}
		//echo ' bucket name'.$name;
		$bucketName = self::toASCII($name);
		//echo ' bucket name'.$bucketName;
		
		$bucketName = str_replace("_","-",$bucketName);
		$bucketName = str_replace("..",".",$bucketName);
		$bucketName = trim($bucketName," .-");
		$bucketName = strtolower($bucketName);
		$bucketName = substr($bucketName,0,63);
		return($bucketName);
	}
	
	public static function toASCII($utf8_sentence)
	{
			//transliterate
			$trans_sentence = iconv('UTF-8', 'ASCII//TRANSLIT', $utf8_sentence);		
		    return $trans_sentence;
	}
	
	public static function testExtension($name, $allowedExtensions = array(), $allowPHP = false)
	{
		$parts = explode(".",$name);
		
		if(!$allowPHP)
		{
			foreach($parts as &$part)
			{
				$part = strtolower($part);
				if($part == "php")
				{
					return false; //weed out php double extensions
				}
			}
		}
		
		foreach($allowedExtensions as &$e)
		{
			$e = strtolower($e);
		}
		$end = end($parts);
		if(in_array($end, $allowedExtensions))
		{
			return true;
		}
			
		return false;

	}


    public static function getAllowedExtensions($params = null)
	{
		if(is_null($params))
		{
			$params = JComponentHelper::getParams('com_amazons3');
		}
		$extensions = $params->get("allowedExtensions","jpg,jpeg,png,gif,mp3,flv,avi,wav,mov,zip,pdf,txt,swf,doc,docx,xls,ogg,mp4,wma,wmv,mpeg");
		$extensions = str_replace(array(" ","."),"",$extensions);
		$allowedExtensions = explode(",",$extensions);
		return $allowedExtensions;
	
	}
	
	public static function listPackages($params = null)
	{
		$db = JFactory::getDBO();
		$query = $db->getQuery(true);
		$query->select($db->quoteName(array('id','title')))->from($db->quoteName('#__amazons3_packages'))->order($db->quoteName('title').' ASC');
		$db->setQuery($query);
		
		if($results = $db->loadObjectList())
		{
			return $results;
		}
		return array();
		
	}
	
	public static function getFileCodes($package = 0, $file = 0)
	{
		$db = JFactory::getDBO();
		$nullDate = $db->quote($db->getNullDate());
		$date = JFactory::getDate();
		$nowDate = $db->quote($date->toSql());
		
		
		
		
		$query = $db->getQuery(true);		
		
		if($package > 0)
		{
			$query->select($db->quoteName(array('a.code','a.description','a.title','b.catid','a.params')))->from($db->quoteName('#__amazons3_packages_files','a'));			
			$query->join("LEFT", $db->quoteName("#__amazons3_packages", "b")." ON ". $db->quoteName("a.package")." = ".$db->quoteName("b.id"));
			
		}
		else
		{
			$query->select($db->quoteName(array('a.code','a.description','a.title','a.params')))->from($db->quoteName('#__amazons3_packages_files','a'));
		}

		$where = array();
		$where[] = $db->quoteName('a.published').' = \'1\'';
		$where[] = '('. $db->quoteName('a.publish_up').' = ' . $nullDate . ' OR '.$db->quoteName('a.publish_up').' <= ' . $nowDate . ')';
		$where[] = '('. $db->quoteName('a.publish_down').' = ' . $nullDate . ' OR '.$db->quoteName('a.publish_down').' >= ' . $nowDate . ')';
		
		if($package > 0)
		{
			$where[] = $db->quoteName('a.package').' = '.$db->quote((int)$package);
			$where[] = $db->quoteName('b.published').' = \'1\'';
			$where[] = '('. $db->quoteName('b.publish_up').' = ' . $nullDate . ' OR '.$db->quoteName('b.publish_up').' <= ' . $nowDate . ')';
			$where[] = '('. $db->quoteName('b.publish_down').' = ' . $nullDate . ' OR '.$db->quoteName('b.publish_down').' >= ' . $nowDate . ')';
			
		}
		
		if($file > 0)
		{
			$where[] = $db->quoteName('a.id').' = '.$db->quote((int)$file);
		}
		
		$query->where($where)->order('a.ordering ASC');
		$db->setQuery($query);
		
		if($results = $db->loadObjectList())
		{
			if($package > 0)
			{
				$badcats = self::getBadCats();
				if(empty($badcats)){return $results;}
				$ret = array();
				foreach($results as $result)
				{
					
					if(!in_array((int)$result->catid, $badcats))
					{
						$ret[] = $result;
					}
				}
				return $ret;
			}
			return $results;
		}
		return array();
		
		
	}
	
	public static function getFileForOrder($package = 0, $code)
	{
		
		if(empty($code))
		{
			return false;
		}
		
		//gets the file by code and package
		$db = JFactory::getDbo();
		// Filter by start and end dates.
		$nullDate = $db->quote($db->getNullDate());
		$date = JFactory::getDate();

		$nowDate = $db->quote($date->toSql());
		
		
			
		if(!empty($package))
		{
			$query = $db->getQuery(true)->select("a.*")->select($db->quoteName("b.catid"))->from($db->quoteName("#__amazons3_packages_files","a"));			
			
			$query->join("LEFT", $db->quoteName("#__amazons3_packages", "b")." ON ". $db->quoteName("a.package")." = ".$db->quoteName("b.id"));
		}
		else
		{
			$query = $db->getQuery(true)->select("a.*")->from($db->quoteName("#__amazons3_packages_files","a"));
		}
		
		$query->where('(a.code = '.$db->quote($code).')');
	
		
		if(!empty($package))
		{
			$query->where('(a.package = '.$db->quote($package).')');
			$query->where('(b.published = '.$db->quote('1').')');
			$query->where('(b.publish_up = ' . $nullDate . ' OR b.publish_up <= ' . $nowDate . ')')
				->where('(b.publish_down = ' . $nullDate . ' OR b.publish_down >= ' . $nowDate . ')');
			
			
		}
		
		$query->where('(a.published = '.$db->quote('1').')');
		
		

		$query->where('(a.publish_up = ' . $nullDate . ' OR a.publish_up <= ' . $nowDate . ')')
			->where('(a.publish_down = ' . $nullDate . ' OR a.publish_down >= ' . $nowDate . ')');
		
		$db->setQuery($query);

		$data = $db->loadObject();

		if (empty($data))
		{
			return false;
		}

		// Check for published state if filter set.
		if ($data->published != 1)
		{
			return false;
		}
		
		if(!empty($package))
		{		
			$badcats = self::getBadCats();
			if(in_array((int)$data->catid, $badcats))
			{
				return false;
			}
		}
		
		
		
		// Convert parameter fields to objects.
		$registry = new JRegistry;
		$registry->loadString($data->params);

		$data->params = self::getParams();
		$data->params->merge($registry);	
		
		return $data;

		
	}
	
	
	public static function getBadCats()
	{
		static $badcats;
		if(isset($badcats))
		{
			return $badcats;
		}
		
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		
		$subquery = ' SELECT cat.id as id FROM #__categories AS cat JOIN #__categories AS parent ';
		$subquery .= 'ON cat.lft BETWEEN parent.lft AND parent.rgt ';
		$subquery .= 'WHERE parent.extension = ' . $db->quote('com_amazons3');
		$subquery .= ' AND parent.published <= 0 GROUP BY cat.id';
		
		$db->setQuery($subquery);
		if($result = $db->loadColumn())
		{
			$badcats = $result;
		}
		else
		{
			$badcats = array();
		}
		
		return $badcats;
		
	}
	
    public static function checkFileExists($object, $bucket)
	{
		$searchString = '"location":{"bucket":"'.$bucket.'","object":"'.$object.'"}';
		$db = JFactory::getDBO();
		$query = $db->getQuery(true);
		$query->select($db->quoteName('id'))->from($db->quoteName('#__amazons3_packages_files'));
		$query->where($db->quoteName('source') . ' = ' . $db->quote('amazons3'));
		$query->where($db->quoteName('params').' LIKE '.$db->quote('%' . $db->escape($searchString).'%'));
		try{
			$db->setQuery($query);
			if($result = $db->loadResult())
			{
				return true;
			}
			else
			{
				return false;
			}
		} catch(Exception $e){return false;}
	}
	
	
	public static function makeNewFile($object, $bucket)
	{
		JTable::addIncludePath(JPATH_ADMINISTRATOR.'/components/com_amazons3/tables/');		
		JModelLegacy::addIncludePath(JPATH_ADMINISTRATOR.'/components/com_amazons3/models/', 'Amazons3');
		$model = JModelLegacy::getInstance('File', 'Amazons3Model', array('ignore_request' => true));
        $jinput = JFactory::getApplication()->input;	
		
		$data = $jinput->get('jform', array(),'array');		
		$data["id"] = 0;
		$data["title"] = basename($object);
		$data["source"] = "amazons3";
		
		$params = array();
		//$params["location"] = '{"bucket":"'.$bucket.'","object":"'.$object.'"}';	
		$location = new stdClass();
		$location->bucket = $bucket;
		$location->object = $object;
		$params["location"] = $location;
		$params["downloadType"] = $data["downloadType"];
		$params["dlocation"] = "";
		$data["params"] = $params;
		
		if($model->save($data))
		{
		
			return true;
		}
		else
		{
			return false;
		}
	}
	
	
	


}

