<?php
/**
 * @copyright	Copyright (C) 2015 Spiral Scripts. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */

// no direct access
defined('_JEXEC') or die;
require(JPATH_SITE.'/components/com_amazons3/aws/aws_2.8.19.phar');
use Aws\S3\S3Client;
use Aws\S3\Model\ClearBucket;


abstract class AmazonS3APIHelper
{
	static $error = null;
	static $client = null;
	
	public static function getClient($params)
	{
		//gets a new client
		try{
			$client = S3Client::factory(array(
				'credentials' => array(
					'key'      => $params->get('access_key',''),
					'secret'   => $params->get('secret_key',''),
					'signature' => 'v4'
				)
			));		
		} catch(Exception $e) {
			self::$error = $e;
			return false;
			//echo 'Caught exception: ',  $e->getMessage(), "\n";
		}
		
		self::$client = $client;
		return $client;
	}
	
	public static function getErrorMessage()
	{
		if(is_null(self::$error))
		{
			return "";
		}
		else
		{
			return self::$error->getMessage();
		}
		
	}
	
	public static function createBucket($params, $name, $region = "us-east-1", $acl = "private")
	{
		if(is_null(self::$client))
		{
			$client = self::getClient($params);
		}
		else
		{
		    $client = self::$client;	
		}
		
		if(!$client){ return false; }
		
		try{

			$result = $client->createBucket(array(
				'Bucket'             => $name,
				'LocationConstraint' => $region,
				'ACL'				 => $acl
			));
			
			$client->waitUntil('BucketExists', array('Bucket' => $name));
			
		} catch(Exception $e) {
			self::$error = $e;
			return false;
		}
		
		return true;
		
	}
	
	
	public static function deleteBucket($params, $name)
	{
		if(is_null(self::$client))
		{
			$client = self::getClient($params);
		}
		else
		{
		    $client = self::$client;	
		}
		
		if(!$client){ return false; }
		
		try{
			$clear = new ClearBucket($client, $name);
			$clear->clear();			

			$result = $client->deleteBucket(array(
				'Bucket' => $name
			));
			
			$client->waitUntil('BucketNotExists', array('Bucket' => $name));
			
		} catch(Exception $e) {
			self::$error = $e;
			return false;
		}
		
		return true;
		
		
	}
	
	public static function listBuckets($params)
	{
		if(is_null(self::$client))
		{
			$client = self::getClient($params);
		}
		else
		{
		    $client = self::$client;	
		}
		
		if(!$client){ return false; }
		
		try{
			$buckets = $client->listBuckets();
		} catch(Exception $e) {
			self::$error = $e;
			return false;
		}
		
		return $buckets;
		
		
	}
	
	public static function listObjects($params, $bucket, $options = array())
	{
		if(is_null(self::$client))
		{
			$client = self::getClient($params);
		}
		else
		{
		    $client = self::$client;	
		}
		
		
		if(!$client){ return false; }
		
		$options["Bucket"] = $bucket;
				
		try{			
			//$iterator = $client->getIterator('ListObjects', $options);
			$result  =  $client->listObjects($options);
			
		} catch(Exception $e) {
			self::$error = $e;
			return false;
		}
		
		return $result;

	}
	
	public static function getCommonPrefixes($params, $bucket)
	{
		if(is_null(self::$client))
		{
			$client = self::getClient($params);
		}
		else
		{
		    $client = self::$client;	
		}
		if(!$client){ return false; }
		
		$delimiter = $params->get("delimiter","/");
		
		try{
			$result  =  $client->listObjects(array(
					"Bucket" => $bucket,
					"Delimiter" => $delimiter	
			));
			
		} catch(Exception $e) {
			self::$error = $e;
			return false;
		}
		
		return $result["CommonPrefixes"];
		
		
	}
	
	
	public static function deleteObjects($params, $bucket, $objects)
	{
		if(is_null(self::$client))
		{
			$client = self::getClient($params);
		}
		else
		{
		    $client = self::$client;	
		}
				
		if(!$client){ return false; }
		
		try{
			$result = $client->deleteObjects(array(
				// Bucket is required
				'Bucket' => $bucket,
				// Key is required
				'Objects' => $objects
			));			
		} catch(Exception $e) {
			self::$error = $e;
			return false;
		}
		return true;
		
	}
	
	public static function upload($params, $bucket, $objectName, $pathToFile)
	{
		if(is_null(self::$client))
		{
			$client = self::getClient($params);
		}
		else
		{
		    $client = self::$client;	
		}
				
		if(!$client){ return false; }
		try{
			// Upload an object by streaming the contents of a file
			// $pathToFile should be absolute path to a file on disk
			$result = $client->putObject(array(
				'Bucket'     => $bucket,
				'Key'        => $objectName,
				'SourceFile' => $pathToFile
			));
			
			// We can poll the object until it is accessible
			$client->waitUntil('ObjectExists', array(
				'Bucket' => $bucket,
				'Key'    => $objectName
			));			
			
		} catch(Exception $e) {
			self::$error = $e;
			return false;
		}
		return true;
	}
	
	public static function download($params, $bucket, $object)
	{
	    if($params->get('allowStreaming','no') == 'yes')
		{
			self::streamDownload($params, $bucket, $object);
			return;
		}
		if(is_null(self::$client))
		{
			$client = self::getClient($params);
		}
		else
		{
		    $client = self::$client;	
		}
		if(!$client){ return false; }
		
		
		set_time_limit(0);
		do
		{
		  //clean all output buffers
		} while(@ob_end_clean());
		
		$timeout = (int)$params->get('timeout',10);
		if($timeout == 0)
		{
			$timeout = 10;
		}
				
		if(!$client){ return false; }
		
		
		if($params->get("downloadType","attachment") == "inline")
		{
			$fileTypes = self::getFileMimeTypes();
			$filename = basename($object);
			$extension = self::extension($filename);
			
			if(array_key_exists($extension,$fileTypes))
			{
				//the file extension is in the list of files that can display
				$contentType = $fileTypes[$extension];
			}
			
			
			try{
				
				$command = $client->getCommand('GetObject', array(
					'Bucket' => $bucket,
					'Key' => $object,
					'ResponseContentDisposition' => 'inline; filename="'.basename($object).'"',
					'ResponseContentType' => $contentType
				));
				
				// Create a signed URL from the command object that will last for
				// timeout minutes from the current time
				$signedUrl = $command->createPresignedUrl('+'.$timeout.' minutes');
				header('Location: '.$signedUrl);
				
				
			} catch(Exception $e) {
				self::$error = $e;
				echo 'Caught exception: ',  $e->getMessage(), "\n";
				return false;
			}
			
			jExit();
		}
		else
		{
			try{

				$command = $client->getCommand('GetObject', array(
					'Bucket' => $bucket,
					'Key' => $object,
					'ResponseContentDisposition' => 'attachment; filename="'.basename($object).'"'
				));
				
				// Create a signed URL from the command object that will last for
				// timeout minutes from the current time
				$signedUrl = $command->createPresignedUrl('+'.$timeout.' minutes');
			
			} catch(Exception $e) {
				self::$error = $e;
				echo 'Caught exception: ',  $e->getMessage(), "\n";
				return false;
			}
			
			header('Location: '.$signedUrl);
			jExit();
			
			
		}
		
	}
	
	public static function streamDownload($params, $bucket, $object)
	{
		if(is_null(self::$client))
		{
			$client = self::getClient($params);
		}
		else
		{
		    $client = self::$client;	
		}
		if(!$client){ return false; }
		
		$client->registerStreamWrapper();
		
		set_time_limit(0);
		do
		{
		  //clean all output buffers
		} while(@ob_end_clean());
		
		$filename = basename($object);
		$extension = self::extension($filename);
		
		
        jimport('joomla.environment.browser');
        $browser = JBrowser::getInstance();
        $browserType = $browser->getBrowser();
        if($browserType == 'msie')
        {
			$filename = preg_replace ('/\./', '%2e', $filename, substr_count ($filename, '.') - 1);			
        }	
		
		$disposition = "attachment";
		$contentType = "application/octet-stream";
		
		if($params->get("downloadType","attachment") == "inline")
		{
			$fileTypes = self::getFileMimeTypes();
			if(array_key_exists($extension,$fileTypes))
			{
				//the file extension is in the list of files that can display
				$contentType = $fileTypes[$extension];
				$disposition = "inline";
			}
		}
		
		$range = 0;
		$size = filesize("s3://{$bucket}/{$object}");
		$end = $size - 1;
		if ($size == 0) {
			jExit();
		}

		header('Cache-Control: public');
		header ("Content-Transfer-Encoding: binary");
		header ('Content-Type: ' . $contentType);
		header("Content-Disposition: ".$disposition."; filename=\"".$filename."\"");				
		header ("Accept-Ranges: bytes");
		header("Content-Length: $size");
		header ("Content-Range: bytes 0-$end/$size");
		
		$speed = (int)$params->get("speed",100);
		if($speed == 0)
		{
			$speed = 100;
		}
		
		$buffer = 1024;
		$delay = 1000000/$speed; //delay in microseconds
		if($delay == 0)
		{
			$delay = 10000;
		}
		
		
		// Open a stream in read-only mode
		if (!($stream = fopen("s3://{$bucket}/{$object}", 'rb'))) {
			jExit('Could not open stream for reading');
		}
		 
		// Check if the stream has more data to read
		while (!feof($stream)) {
			echo fread($stream, $buffer);
			flush();
			usleep($delay);

		}
		// close the stream resource when you're done with it
		fclose($stream);		
		
		jExit();
		
	}
	
	public static function extension($str){
		$str=explode(".",$str);
		$str=strtolower(end($str));
		return $str;
   }
	
	
	public static function getFileMimeTypes()
	{
		$fileTypes = array();
		
		$fileTypes['swf'] = 'application/x-shockwave-flash';
		$fileTypes['pdf'] = 'application/pdf';
		$fileTypes['exe'] = 'application/octet-stream';
		$fileTypes['zip'] = 'application/zip';
		$fileTypes['doc'] = 'application/msword';
		$fileTypes['docx'] = 'application/msword';
		$fileTypes['xls'] = 'application/vnd.ms-excel';
		$fileTypes['ppt'] = 'application/vnd.ms-powerpoint';
		$fileTypes['rar'] = 'application/rar';
		$fileTypes['txt'] = 'text/plain';
		
		
		$fileTypes['gif'] = 'image/gif';
		$fileTypes['png'] = 'image/png';
		$fileTypes['jpeg'] = 'image/jpg';
		$fileTypes['jpg'] = 'image/jpg';

		$fileTypes['ra'] = 'audio/x-pn-realaudio';
		$fileTypes['ram'] = 'audio/x-pn-realaudio';
		$fileTypes['ogg'] = 'audio/ogg';
		$fileTypes['mp3'] = 'audio/mpeg';
		$fileTypes['wma'] = 'audio/x-ms-wma';
		$fileTypes['wav'] = 'audio/wav';				

		$fileTypes['wmv'] = 'video/x-ms-wmv';
		$fileTypes['avi'] = 'video/x-msvideo';
		$fileTypes['asf'] = 'video/x-ms-asf';
		$fileTypes['divx'] = 'video/x-msvideo';

		$fileTypes['mpeg'] = 'video/mpeg';
		$fileTypes['mpg'] = 'video/mpeg';
		$fileTypes['mpe'] = 'video/mpeg';
		$fileTypes['mp4'] = 'video/mp4';
		$fileTypes['flv'] = 'video/x-flv';
		
		
		$fileTypes['mov'] = 'video/quicktime';
		$fileTypes['3gp'] = 'video/quicktime';
		$fileTypes['m4a'] = 'video/quicktime';
		$fileTypes['aac'] = 'video/quicktime';
		$fileTypes['m3u'] = 'video/quicktime';	
		
		return $fileTypes;
	}

}

?>