<?php
/**
 * @copyright	Copyright (C) 2015 Spiral Scripts. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */

// no direct access
defined('_JEXEC') or die;

abstract class DropboxapiHelper
{
   public static $rpcEndpoint = "https://api.dropboxapi.com/2";
   public static $authEndpoint = "https://api.dropboxapi.com";   
   public static $contentEndpoint = "https://content.dropboxapi.com";
   public static $lastErrorCode = 0;
   public static $lastCurlError = 0;
   

   public static function postCurl($url, $data = "", $access_token = "")
   {
	   if(empty($data))
	   {
		  return false;   
	   }
	   $ch = curl_init($url);
	   curl_setopt($ch, CURLOPT_POST, 1);
	   $headers = array();
	   if(!empty($data))
	   {
		 curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
		 if(!empty($access_token))
		 {
			 //if $access token is empty then this is authentication call and for some reason it won't access json
			 $headers[] = 'Content-Type: application/json';
		 }
	   }
	   
	   if(!empty($access_token))
	   {
		   $headers[] = 'Authorization: Bearer '. $access_token;
	   }
	   
	   curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
	   curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	   $result = curl_exec($ch);
	   self::$lastCurlError = curl_errno($ch);
	   if(!self::$lastCurlError)
	   {
		  $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		  if((int)$http_code != 200)
		  {
			  self::$lastErrorCode = (int)$http_code;
			  $errors = json_decode($result);
			  $app = JFactory::getApplication();
			  if(isset($errors->error))
			  {
			    $app->enqueueMessage($errors->error . ' : '. $errors->error_description);
			  }
			  else
			  {
				$app->enqueueMessage($result);
			  }
			  $return = false;
		  }
		  else
		  {
			  $return = $result;
		  }
	   }
	   else
	   {
		   $return = false;
	   }


	   
	   curl_close($ch);
	   return $return;
	   
   }
   
   public static function keyCheck($params)
   {
	   //check that the keys are not empty
	   $access_key = $params->get("access_key", false);
	   $secret_key = $params->get("secret_key", false);
	   
	   if(!$access_key || !$secret_key)
	   {
		   return false;
	   }
	   else
	   {
		   return true;   
	   }
	   
	   
   }
   
   public static function oauth2Token($params, $auth_code)
   {
	   $data = array();
	   $data["code"] = $auth_code;
	   $data["grant_type"] = 'authorization_code';
	   $data["client_id"] = $params->get("access_key");
	   $data["client_secret"] = $params->get("secret_key");
	   
	   $encoded_data = http_build_query($data);
	   $url = self::$authEndpoint.'/oauth2/token';
	   
	   $response = self::postCurl($url, $encoded_data);
	   return $response;
	   
   }
   
   public static function authTokenRevoke($token)
   {
	   $url = self::$rpcEndpoint.'/auth/token/revoke';
	   self::postCurl($url,"", $token);
	   //there shouldn't be a response from this
   }
   
   public static function filesListFolder($token, $options = array())
   {
	   $path = "";
	   if(isset($options["path"]))
	   {
		   $path = $options["path"];
	   }
	   
	   $data = new stdClass();
	   $data->path = $path;
	   $data->recursive = false;
	   $encoded_data = json_encode($data);
	   $url = self::$rpcEndpoint.'/files/list_folder';
	   
	   $response = self::postCurl($url, $encoded_data, $token);
	   return $response;
	   	   
   }
   
   public static function filesSearch($token, $options = array())
   {
	   
	   if(empty($options["query"]))
	   {
		  $response = self::filesListFolder($token, $options);
		  return $response;
	   }
	   
	   $encoded_data = json_encode($options);
	   $url = self::$rpcEndpoint.'/files/search';
	   
	   $response = self::postCurl($url, $encoded_data, $token);
	   return $response;	   
	   
   }
   
   public static function filesListFolderContinue($token, $options = array())
   {
	  $cursor = $options["cursor"];
	  if(empty($cursor))
	  {
		 return false;
	  }
	  
	  $encoded_data = json_encode($options);
	  $url = self::$rpcEndpoint.'/files/list_folder/continue';
	  
	  $response = self::postCurl($url, $encoded_data, $token);
	  return $response;
	   
	   
   }
   
   public static function filesDelete($token, $options = array())
   {
	   if(isset($options["path"]))
	   {
		   $encoded_data = json_encode($options);
	   }
	   else
	   {
		   return false;   
	   }
	   
	   
	   $url = self::$rpcEndpoint.'/files/delete';
	   
	   $response = self::postCurl($url, $encoded_data, $token);
	   return $response;	   
   }
   
   
   public static function filesDeleteBatch($token, $options = array())
   {
	   if(isset($options["entries"]))
	   {
		   $encoded_data = json_encode($options);
	   }
	   else
	   {
		   return false;   
	   }
	   
	   
	   $url = self::$rpcEndpoint.'/files/delete_batch';
	   
	   $response = self::postCurl($url, $encoded_data, $token);
	   return $response;	   
   }
   
   public static function filesDeleteBatchCheck($token, $options = array())
   {
	   if(isset($options["async_job_id"]))
	   {
		   $encoded_data = json_encode($options);
	   }
	   else
	   {
		   return false;   
	   }
	   
	   
	   $url = self::$rpcEndpoint.'/files/delete_batch/check';
	   
	   $response = self::postCurl($url, $encoded_data, $token);
	   return $response;	   
   }   
   
	public static function filesUpload($token, $src, $options = array())
	{
		$url = "https://content.dropboxapi.com/2/files/upload";
		$ch = curl_init($url);
		$aPostData = $options;
		
		//$filename = basename($options["path"]);
		//$file = self::curlFileCreate($src,"",$filename);
		//$data = array("file" => $file);
		
		$fp = fopen($src, 'rb');
        $filesize = filesize($src);
		$data = fread($fp, $filesize);
		
		
		$aOptions = array(
			CURLOPT_POST => true,
			CURLOPT_HTTPHEADER => array('Content-Type: application/octet-stream', 
			  'Authorization: Bearer ' . $token,
			  'Dropbox-API-Arg: ' . json_encode($aPostData)
			) ,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_POSTFIELDS => $data
		);

		curl_setopt_array($ch, $aOptions);
		$result = curl_exec($ch);		

	   self::$lastCurlError = curl_errno($ch);
	   if(!self::$lastCurlError)
	   {
		  $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		  if((int)$http_code != 200)
		  {
			  self::$lastErrorCode = (int)$http_code;
			  $errors = json_decode($result);
			  $app = JFactory::getApplication();
			  if(isset($errors->error_summary))
			  {
			    $app->enqueueMessage($errors->error_summary, "warning");
			  }
			  else
			  {
			    $app->enqueueMessage($result, "warning");
			  }
			  $return = false;
		  }
		  else
		  {
			  $return = $result;
		  }
	   }
	   else
	   {
		   $return = false;
	   }


	   
	   curl_close($ch);
	   return $return;	 
	 
	}
	
   public static function filesGetThumbnail($token, $options = array())
   {
	   if(!isset($options["path"]))
	   {
		   return false;
	   }
	   
		$url = "https://content.dropboxapi.com/2/files/get_thumbnail";
		$curl = curl_init($url);
		$aOptions = array(
			CURLOPT_POST => true,
			CURLOPT_HTTPHEADER => array('Content-Type: ', // To force empty Content-type
			  'Authorization: Bearer ' . $token,
			  'Dropbox-API-Arg: ' . json_encode($options)
			) ,
			CURLOPT_RETURNTRANSFER => true
		);
		
		curl_setopt_array($curl, $aOptions);
		$result = curl_exec($curl);	
	   
	   self::$lastCurlError = curl_errno($curl);
	   if(!self::$lastCurlError)
	   {
		  $http_code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
		  if((int)$http_code != 200)
		  {
			  self::$lastErrorCode = (int)$http_code;
			  $errors = json_decode($result);
			  $app = JFactory::getApplication();
			  if(isset($errors->error_summary))
			  {
			    $app->enqueueMessage($errors->error_summary, "warning");
			  }
			  else
			  {
			    $app->enqueueMessage($result, "warning");
			  }
			  $return = false;
		  }
		  else
		  {
			  $return = $result;
		  }
	   }
	   else
	   {
		   $return = false;
	   }
	   curl_close($curl);
	   return $return;	 
	   
   }
   
   public static function filesDownload($token, $options = array())
   {
	   
		$path = "";
		$pathIsPath = false;
		
		if(isset($options["path_display"]))
		{
		   $path = $options["path_display"];
		   $pathIsPath = true;
		}
		elseif(isset($options["id"]))
		{
			//$path = "id:".$options["id"];
			$path = $options["id"];
			
		}
		
		if(empty($path))
		{
		   return false;
		}
		
		$getHeaders = false;
		
		$url = "https://content.dropboxapi.com/2/files/download";
		$curl = curl_init($url);
		$aPostData = array('path' => $path);
		$aOptions = array(
			CURLOPT_POST => true,
			CURLOPT_HTTPHEADER => array('Content-Type: ', // To force empty Content-type
			  'Authorization: Bearer ' . $token,
			  'Dropbox-API-Arg: ' . json_encode($aPostData)
			) ,
			CURLOPT_RETURNTRANSFER => true
		);
		
		if($getHeaders)
		{
			$aOptions[CURLOPT_HEADER] = true;
		}
		curl_setopt_array($curl, $aOptions);
		
		//$fp = fopen('out.jpg', 'w');
		//fwrite($fp, $result);
		//fclose($fp);
		
		if($getHeaders)
		{
			$result = curl_exec($curl);		
			$header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
			$header = substr($result, 0, $header_size);
			$body = substr($result, $header_size);	
		}
		else
		{
			$body = curl_exec($curl);	
		}
		
		curl_close($curl);
		
		
		$size = strlen($body);
		$contentType = "application/octet-stream";		
		$disposition = "attachment";

		if(isset($options["name"]) && !empty($options["name"]))
		{
			$filename = $options["name"];
		}
		elseif($pathIsPath)
		{
			$filename = basename($path);
		}
		else
		{
		   //what do we do if neither apply?
		    return false;
		}
		
		if(isset($options["disposition"]) && $options["disposition"] == "inline")
		{
			$extension = self::extension($filename);
			$fileTypes = self::getFileMimeTypes();
			if(array_key_exists($extension,$fileTypes))
			{
				//the file extension is in the list of files that can display
				$contentType = $fileTypes[$extension];
				$disposition = "inline";
			}
			
		}
				

		set_time_limit(0);
		do
		{
		  //clean all output buffers
		} while(@ob_end_clean());

		
        /* do simple download */
		header('Content-Description: File Transfer');
		header("Content-Type: ". $contentType);
		header('Expires: 0');
		header('Cache-Control: must-revalidate');
		header('Pragma: public');  
		header("Content-Disposition: ".$disposition."; filename=\"".$filename."\"");
		header('Content-Length: ' . $size);
		
		echo $body;
		
		jexit();	   
	   
   }
   
   public static function filesGetTemporaryLink($token, $options = array())
   {
	   
		$path = "";
		if(isset($options["path_display"]))
		{
		 $path = $options["path_display"];
		}
		elseif(isset($options["id"]))
		{
		  $path = $options["id"];
		}
	  
		if(empty($path))
		{
		   return false;
		}
	  
	   
	   $data = new stdClass();
	   $data->path = $path;
	   $encoded_data = json_encode($data);
	   $url = self::$rpcEndpoint.'/files/get_temporary_link';
	   
	   $responseJSON = self::postCurl($url, $encoded_data, $token);
	   $response = json_decode($responseJSON);
	   return $response->link;
	   	   
   }
   
   public static function filesGetMetadata($token, $options = array())
   {
	   
   }
   
   public static function curlFileCreate($filename, $mimetype, $postname)
   {
	   
	  if (!function_exists('curl_file_create')) {
			   return "@$filename;filename="
				   . ($postname ?: basename($filename))
				   . ($mimetype ? ";type=$mimetype" : '');
	   }
	   else
	   {		   
		  return curl_file_create($filename, $mimetype, $postname); 
	   }
	   
	   
	   
   }
   
	public static function extension($str){
		$str=explode(".",$str);
		$str=strtolower(end($str));
		return $str;
   }
   

	public static function getFileMimeTypes()
	{
		$fileTypes = array();
		
		$fileTypes['swf'] = 'application/x-shockwave-flash';
		$fileTypes['pdf'] = 'application/pdf';
		$fileTypes['exe'] = 'application/octet-stream';
		$fileTypes['zip'] = 'application/zip';
		$fileTypes['doc'] = 'application/msword';
		$fileTypes['docx'] = 'application/msword';
		$fileTypes['xls'] = 'application/vnd.ms-excel';
		$fileTypes['ppt'] = 'application/vnd.ms-powerpoint';
		$fileTypes['rar'] = 'application/rar';
		$fileTypes['txt'] = 'text/plain';
		
		
		$fileTypes['gif'] = 'image/gif';
		$fileTypes['png'] = 'image/png';
		$fileTypes['jpeg'] = 'image/jpg';
		$fileTypes['jpg'] = 'image/jpg';

		$fileTypes['ra'] = 'audio/x-pn-realaudio';
		$fileTypes['ram'] = 'audio/x-pn-realaudio';
		$fileTypes['ogg'] = 'audio/ogg';
		$fileTypes['mp3'] = 'audio/mpeg';
		$fileTypes['wma'] = 'audio/x-ms-wma';
		$fileTypes['wav'] = 'audio/wav';				

		$fileTypes['wmv'] = 'video/x-ms-wmv';
		$fileTypes['avi'] = 'video/x-msvideo';
		$fileTypes['asf'] = 'video/x-ms-asf';
		$fileTypes['divx'] = 'video/x-msvideo';

		$fileTypes['mpeg'] = 'video/mpeg';
		$fileTypes['mpg'] = 'video/mpeg';
		$fileTypes['mpe'] = 'video/mpeg';
		$fileTypes['mp4'] = 'video/mp4';
		$fileTypes['flv'] = 'video/x-flv';
		
		
		$fileTypes['mov'] = 'video/quicktime';
		$fileTypes['3gp'] = 'video/quicktime';
		$fileTypes['m4a'] = 'video/quicktime';
		$fileTypes['aac'] = 'video/quicktime';
		$fileTypes['m3u'] = 'video/quicktime';	
		
		return $fileTypes;
	}



}

?>