<?php
/**
 * @author Fiona Coulter		
 * @copyright Spiral Scripts
 * @license GPL		
 */
 
 
defined("_JEXEC") or die("Restricted access");

require_once(JPATH_ADMINISTRATOR.'/components/com_amazons3/defines.php');

$s3_path = JPATH_ADMINISTRATOR.'/components/com_amazons3/helpers/';
JLoader::register('AmazonS3APIHelper',$s3_path.'amazons3api.php');
JLoader::register('AmazonS3Helper',$s3_path.'amazons3.php');
JLoader::register('DropboxHelper',$s3_path.'dropbox.php');
JLoader::register('DropboxapiHelper',$s3_path.'dropboxapi.php');




class Amazons3OrderAdminHelper
{
	
	public static function startPackageDownload($package = 0, $filecode = "")
	{
		//takes file code, and optional package parameter and starts download
		if(!empty($filecode))
		{
			$file = self::getFileForOrder($package, $filecode);
			self::startDownload($file);
		}
		
	}

	
	public static function startDownload($file)
	{
		if(!$file)
		{
		   	JError::raiseError( 403, 'File is not available' );
			return;
		}
		
		$source = $file->source;
		JModelLegacy::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_amazons3/models/');
		$model = JModelLegacy::getInstance('File', 'Amazons3Model', array('ignore_request' => true));
		
		
		
		switch($source)
		{
			case "amazons3":
				$location = (array)$file->params->get("location");
				$params = AmazonS3Helper::getParams();	
				$params->merge($file->params);
								
				if(isset($location["bucket"]) && isset($location["object"]) && !empty($location["bucket"]) && !empty($location["object"]))
				{
					  $model->hit($file->id);
					  AmazonS3APIHelper::download($params, $location["bucket"],$location["object"]);
				}
				break;
				
			case "dropbox": 
				$dlocation = $file->params->get("dlocation");
				$params = AmazonS3Helper::getParams(DROPBOX_KEY_ID);		//id of 2 for dropbox keys
				$options = json_decode($dlocation, true);
				
				$params->merge($file->params);
				$options["disposition"] = $params->get("downloadType","attachment");
				
				$downloadType = $params->get("dropbox_download_type", "standard");
				$token = DropboxHelper::getToken($params);
								
				$model->hit($file->id);
				if($downloadType == "standard")
				{
					DropboxapiHelper::filesDownload($token, $options);
					
				}
				elseif($downloadType == "redirect")
				{
					$link = DropboxapiHelper::filesGetTemporaryLink($token, $options);
					$app = JFactory::getApplication();
					$app->redirect($link);
					return;
					
				}			
		}
				
		jexit();
	}
	
	public static function startFileDownload($id = 0)
	{
		//takes file id, starts download
		    if(empty($id))
			{
				return;
			}
			$file = self::getFileById($id);	
			if(!$file){return; }
			self::startDownload($file);
	}
	
	
	public static function getSiteURL()
	{		
		$baseurl = JURI::root();
		$baseurl= str_replace('/administrator','',$baseurl);	
		
		$params = JComponentHelper::getParams('com_amazons3');
		$text = $params->get('siteURL',$baseurl);
		$text = trim($text);
		return $text;		
	}
	
	
	
	public static function sendCustomerEmail($data)
	{
		
		//before we get to this point we should have checked order status to make sure it is valid
		$email = $data["email"];
		
		
		if(empty($email))
		{
			//try looking at Joomla user
			if(isset($data["user_id"]) && !empty($data["user_id"]))
			{
				$user = JFactory::getUser($data["user_id"]);
				if(isset($user->email) && !empty($user->email))
				{
					$email = $user->email;
				}
			}
		}
		
		if(empty($email))
		{
		  return false;	
		}
		
		$data["siteURL"] = self::getSiteURL();
		
		//now send the email
		
		$mailer = JFactory::getMailer();
		$config = JFactory::getConfig();
		$sender = array( 
			$config->get( 'mailfrom' ),
			$config->get( 'fromname' ) 
		);
 
		$mailer->setSender($sender);	
		$mailer->addRecipient($email);
		
		if(isset($data["subject"]) && !empty($data["subject"]))
		{
			$subject = $data["subject"];
		}
		else
		{
			$subject = JText::_('AMAZONS3_EMAIL_SUBJECT');
		}
		$mailer->setSubject($subject);
		$mailer->isHTML(true);
		$mailer->Encoding = 'base64';	
		if(isset($data["layout"]) && !empty($data["layout"]))
		{
			$layoutPath = self::getSublayoutPath($data["layout"]);
			ob_start();
			include($layoutPath);
			$body = ob_get_contents();
			ob_end_clean();
		}
		elseif(isset($data["message"]) && !empty($data["message"]))
		{
			$body = $data["message"];
		}
		else
		{
			$layoutPath = self::getSublayoutPath("email.php");
			ob_start();
			include($layoutPath);
			$body = ob_get_contents();
			ob_end_clean();						
		}
		$mailer->setBody($body);
		
		$send = $mailer->Send();
		if ( $send !== true ) {
			return false;
		} else {
			return true;
		}		
		
	}
	
	public static function getSublayoutPath($layout)
	{
		$file = basename($layout);
		$layoutPath = JPATH_SITE.'/components/com_amazons3/sublayouts/'.$file;
		if(!file_exists($layoutPath))
		{
		  return false;	
		}
		
		
		$mainframe = JFactory::getApplication();
		if(file_exists(JPATH_SITE.'/templates/'.$mainframe->getTemplate().'/html/com_amazons3/sublayouts/'.$file)){
			$filepath = JPATH_SITE.'/templates/'.$mainframe->getTemplate().'/html/com_amazons3/sublayouts/'.$file;
		} 
		else 
		{
			$filepath = $layoutPath;
		}
        return $filepath;
		
		
	}
	
	public static function getFileById($id)
	{
		//gets the file by id
		$db = JFactory::getDbo();
		// Filter by start and end dates.
		$nullDate = $db->quote($db->getNullDate());
		$date = JFactory::getDate();
		$nowDate = $db->quote($date->toSql());
		
		$query = $db->getQuery(true)->select("a.*")->from($db->quoteName("#__amazons3_packages_files","a"));
		$query->where('(a.id = '.intval($id).')');
		$query->where('(a.published = '.$db->quote('1').')');
		
		

		$query->where('(a.publish_up = ' . $nullDate . ' OR a.publish_up <= ' . $nowDate . ')')
			->where('(a.publish_down = ' . $nullDate . ' OR a.publish_down >= ' . $nowDate . ')');
		
		$db->setQuery($query);

		$data = $db->loadObject();

		if (empty($data))
		{
			return false;
		}
		
				
		// Convert parameter fields to objects.
		$registry = new JRegistry;
		$registry->loadString($data->params);
		$data->params = $registry;
		
		return $data;
		
	}
	
	
	
	public static function getFileForOrder($package = 0, $code)
	{
		//returns the file information for given file code and optional package id
		if(empty($code))
		{
			return false;
		}
		
		return AmazonS3Helper::getFileForOrder($package, $code);
		
	}

	public static function getFileCodes($package = 0, $file = 0)
	{	
	    //returns file code, title, params for package id or file id
		return AmazonS3Helper::getFileCodes($package, $file);		
	}

		
	
	
}