<?php
/**
 * @version		1.0.0 as3 file manager $
 * @package		as3 file manager
 * @copyright	Copyright © Inspiration Web Design http://www.spiralscripts.co.uk 2016 - All rights reserved.
 * @license		GNU/GPL
 * @author		Fiona Coulter
 * @author mail	joomla - at - iswebdesign.co.uk
 * @website		http://www.spiralscripts.co.uk
 *
 **/

// no direct access
defined('_JEXEC') or die('Restricted access');

jimport( 'joomla.application.component.model' );
JLoader::register('AmazonS3Helper', JPATH_COMPONENT_ADMINISTRATOR.'/helpers/amazons3.php');

class Amazons3ModelAmazons3 extends JModelLegacy
{
	/*
	  This model is used for the storage and retrieval of secret keys. The type of the key is indicated by id, by default it is assumed to be the Amazon S3 key, which has an index of 1
	*/
	/**
	 * Constructor that retrieves the ID from the request
	 *
	 * @access	public
	 * @return	void
	 */
	public $validHashValues = array('SHA1','SHA2','MD5');
	 
	function __construct()
	{
		parent::__construct();
		$this->setId(1); 
	}
	
	function setId($id)
	{
		$this->_id		= $id;
		$this->_data	= null;
	}
	
	protected function populateState()
	{
		$app = JFactory::getApplication('administrator');

		// Load the User state.
		$pk = $app->input->getInt('id',1); //if it is not specified, we use value 1
		$this->setState('amazons3.id', $pk);
		
		// Load the parameters.
		$params = JComponentHelper::getParams('com_amazons3');
		$this->setState('params', $params);
	}
	
	
	
	
	public function getData()
	{
		$word = self::getSecretWord();		
		$params = $this->getState('params');
		$id = (int)$this->getState('amazons3.id');
		$hash = $params->get("hash_type","SHA2");
		if(!in_array($hash, $this->validHashValues))
		{
			$hash = "SHA2";
		}
		
		
		
		try{
			$db = JFactory::getDBO();
			$query = $db->getQuery(true);
			if($hash == "SHA2")
			{		  
			
			  $query->select(array("id","keytype","publickey","data","AES_DECRYPT( secretkey, UNHEX(".$hash."(".$db->quote($word).",512))) AS skey", "AES_DECRYPT( encrypted_data, UNHEX(".$hash."(".$db->quote($word).",512))) AS edata"));
			}
			else
			{
			  $query->select(array("id","keytype","publickey","data","AES_DECRYPT( secretkey, UNHEX(".$hash."(".$db->quote($word)."))) AS skey", "AES_DECRYPT( encrypted_data, UNHEX(".$hash."(".$db->quote($word)."))) AS edata"));				
			}
			
			$query->from("#__amazons3_config");
			$query->where("id=".$id);
			$db->setQuery($query);
			$row = $db->loadObject();
					
			if($row)
			{
				$row->secretkey = base64_decode($row->skey);
				$row->encrypted_data = base64_decode($row->edata);
				return $row;
			}
			else
			{
				return false;	
			}
		} catch(Exception $e){ return false; }
		
	}
	
    public static function getKeys($id = 1)
	{
		
		$keys = AmazonS3Helper::getKeys($id);
		return $keys;
		
	}
	
	function storeExtraData($rawdata, $encrypted = false)
	{
		//the extra data won't normally by stored at the same time as the keys so we have an extra method for this
		if (!JFactory::getUser()->authorise('core.admin', 'com_amazons3')) 
		{
			return false;
		}
        JSession::checkToken() or JSession::checkToken('get') or jexit( 'Invalid Token' );		
		$word = self::getSecretWord();		
		$params = $this->getState('params');
		$id = (int)$this->getState('amazons3.id');
		$hash = $params->get("hash_type","SHA2");
		if(!in_array($hash, $this->validHashValues))
		{
			$hash = "SHA2";
		}
		
		$key = self::recordExists($id);
		
		try{
			
			  // Store it
			  $db = JFactory::getDBO();
			  $query = $db->getQuery(true);
			  
			  if(!$key)
			  {
				  if(!$encrypted)
				  {
					  $columns = array("id","keytype","data");
					  $values = array($id, $db->quote($post["keytype"]), $db->quote($rawdata));
				  }
				  else
				  {
					  if(empty($rawdata))
					  {
						  $columns = array("id","keytype","encrypted_data");
						  $values = array($id, $db->quote($post["keytype"]), "NULL");
					  }
					  else
					  {
						  $columns = array("id","keytype","encrypted_data");
						  
						  $data = base64_encode($rawdata);
						  if($hash == "SHA2")
						  {		  
							  $data = "AES_ENCRYPT(". $db->quote($data).", UNHEX(".$hash."(".$db->quote($word).",512)))";
						  }
						  else
						  {
							  $data = "AES_ENCRYPT(". $db->quote($data).", UNHEX(".$hash."(".$db->quote($word).")))";
						  }
						  
						  $values = array($id, $db->quote($post["keytype"]), $data);
						  
					  }
					  
				  }
				  
				  $query
					  ->insert($db->quoteName('#__amazons3_config'))
					  ->columns($db->quoteName($columns))
					  ->values(implode(',', $values));
					  
				   
				  $db->setQuery($query);
				  
				  if(! $db->execute())
				  {
					return false;	
				  }
				  return true;
				  
				  
			  }
			  else
			  {
				  $query->update($db->quoteName('#__amazons3_config'));
				
				  if(!$encrypted)
				  {
					  $data = $rawdata;
					  $updates = array(
									   $db->quoteName('data')."=".$db->quote($data),							   
									   );
				  }
				  else
				  {
					  if(empty($rawdata))
					  {
						  $updates = array($db->quoteName('encrypted_data').'=NULL');
					  }
					  else
					  {
						$data = base64_encode($rawdata);
						if($hash == "SHA2")
						{		  
							$updates = array("encrypted_data=AES_ENCRYPT(". $db->quote($data).", UNHEX(".$hash."(".$db->quote($word).",512)))");
						}
						else
						{
							$updates = array("encrypted_data=AES_ENCRYPT(". $db->quote($data).", UNHEX(".$hash."(".$db->quote($word).")))");
						}
					  }
					  
				  }
				  $query->set($updates);
				  $conditions = array($db->quoteName("id")."=".$id);
				  $query->where($conditions);
				  $db->setQuery($query);
				  
				  if(! $db->execute())
				  {
					return false;	
				  }
				  return true;
			  
			  }
			
		} catch(Exception $e){ return false; }
			
		
		
	}
	

	
	
	function store()
	{
		//stroes the keys
		if (!JFactory::getUser()->authorise('core.admin', 'com_amazons3')) 
		{
			return false;
		}
		
        $jinput = JFactory::getApplication()->input;		
        JSession::checkToken() or jexit( 'Invalid Token' );		
		$word = self::getSecretWord();		
		$post = $jinput->post->getArray(array('keytype' => 'string', 'publickey' => 'string', 'secretkey' => 'string'));
		
		$keyisempty = true; //if the key is empty we don't store it
		$secretkey = "";
		if(!empty($post['secretkey']))
		{
			$secretkey = base64_encode($post['secretkey']);
			$keyisempty = false;
		}
		
		$params = $this->getState('params');
		$id = (int)$this->getState('amazons3.id');
		
		$hash = $params->get("hash_type","SHA2");
		if(!in_array($hash, $this->validHashValues))
		{
			$hash = "SHA2";
		}
		
		//does the key already exist?
		$key = self::recordExists($id);
		if(!$key)
		{
			try{


			  $db = JFactory::getDbo();
			  $query = $db->getQuery(true);
			  
			  if(!$keyisempty)
			  {
				  $columns = array("id","keytype","publickey","secretkey");
				  if($hash == "SHA2")
				  {		  
					  $values = array($id,$db->quote($post["keytype"]), $db->quote($post["publickey"]),"AES_ENCRYPT(". $db->quote($secretkey).", UNHEX(".$hash."(".$db->quote($word).",512)))");
				  }
				  else
				  {
					  $values = array($id, $db->quote($post["keytype"]), $db->quote($post["publickey"]),"AES_ENCRYPT(". $db->quote($secretkey).", UNHEX(".$hash."(".$db->quote($word).")))");
				  }
				  
			  }
			  else
			  {
				  $columns = array("id","keytype","publickey");
				  $values = array($id, $db->quote($post["keytype"]), $db->quote($post["publickey"]));
				  
			  }
			  
			  $query
				  ->insert($db->quoteName('#__amazons3_config'))
				  ->columns($db->quoteName($columns))
				  ->values(implode(',', $values));
				  
			   
			  $db->setQuery($query);
			  
			  if(! $db->execute())
			  {
				return false;	
			  }
			  return true;
			  
			} catch(Exception $e){ return false; }
			
		}
		else
		{
		
			try{	
			  // Store it
			  $db = JFactory::getDBO();
			  $query = $db->getQuery(true);
			  $query->update($db->quoteName('#__amazons3_config'));
			  
			  $updates = array(
							   $db->quoteName('keytype')."=".$db->quote($post["keytype"]),							   
							   $db->quoteName('publickey')."=".$db->quote($post["publickey"])
							   );
			  
			  if(!$keyisempty)
			  {
			  
				  if($hash == "SHA2")
				  {		  
					  $updates[] = "secretkey=AES_ENCRYPT(". $db->quote($secretkey).", UNHEX(".$hash."(".$db->quote($word).",512)))";
				  }
				  else
				  {
					  $updates[] = "secretkey=AES_ENCRYPT(". $db->quote($secretkey).", UNHEX(".$hash."(".$db->quote($word).")))";
				  }
			  
			  }
			  $query->set($updates);
			  $conditions = array($db->quoteName("id")."=".$id);
			  $query->where($conditions);
			  $db->setQuery($query);
			  
			  if(! $db->execute())
			  {
				return false;	
			  }
			  return true;
			} catch(Exception $e){ return false; }
		
		}
		
	}
	
	function checkAdmin()
	{
		if (!JFactory::getUser()->authorise('core.admin', 'com_amazons3')) 
		{
			return false;
		}
		else
		{
			return true;
		}
		
		
	}
	
	private static function getSecretWord()
	{
		$word = AmazonS3Helper::getSecretWord();
		return $word;
	}
	
	public static function recordExists($id = 1)
	{
		try{
			$db = JFactory::getDBO();
			$query = $db->getQuery(true);
			
			
			$query->select("*")->from("#__amazons3_config");
			$query->where("id=".(int)$id);
			$db->setQuery( $query );
			if($row = $db->loadObject())
			{
				return true;
			}
			else
			{
				return false;
			}
			
		} catch(Exception $e){ return false; }
			
		
		
	}
	
	



}
?>