<?php
/**
 * @author Fiona Coulter		
 * @copyright Spiral Scripts
 * @license GPL		
 */

defined("_JEXEC") or die("Restricted access");

/**
 * Supports a modal package picker.
 *
 * @package     Amazons3
 * @subpackage  Modal fields
 */
class JFormFieldModal_S3object extends JFormField
{
	/**
	 * The form field type.
	 *
	 * @var		string
	 * @since   1.6
	 */
	protected $type = 'Modal_S3object';

	/**
	 * Method to get the field input markup.
	 *
	 * @return  string	The field input markup.
	 * @since   1.6
	 */
	protected function getInput()
	{
		$params = JComponentHelper::getParams("com_amazons3");
		$availableHosts = $params->get("availableHosts",array("all"));
		
		if(!in_array("all", $availableHosts) && !in_array("amazons3", $availableHosts))
		{
			return "";
		}
		
		
		$allowEdit		= ((string) $this->element['edit'] == 'true') ? true : false;
		$allowClear		= ((string) $this->element['clear'] != 'false') ? true : false;

		// Load language
		JFactory::getLanguage()->load('com_amazons3', JPATH_ADMINISTRATOR);

		// Load the javascript
		JHtml::_('behavior.framework');
		JHtml::_('behavior.modal', 'a.modal');
		JHtml::_('bootstrap.tooltip');

		// Build the script.
		$script = array();

		// Select button script
		$script[] = '	function jSelectObject_'.$this->id.'(bucket, object) {';
		$script[] = '		document.id("'.$this->id.'_bucket").value = bucket;';
		$script[] = '		document.id("'.$this->id.'_object").value = object;';

		if ($allowEdit)
		{
			$script[] = '		jQuery("#'.$this->id.'_edit").removeClass("hidden");';
		}

		if ($allowClear)
		{
			$script[] = '		jQuery("#'.$this->id.'_clear").removeClass("hidden");';
		}

		$script[] = '		SqueezeBox.close();';
		$script[] = '	}';

		// Clear button script
		static $scriptClear;

		if ($allowClear && !$scriptClear)
		{
			$scriptClear = true;

			$script[] = '	function jClearObject(id) {';
			$script[] = '		document.getElementById(id + "_bucket").value = "";';
			$script[] = '		document.getElementById(id + "_object").value = "'.htmlspecialchars(JText::_('COM_AMAZONS3_SELECT_AN_OBJECT', true), ENT_COMPAT, 'UTF-8').'";';
			$script[] = '		jQuery("#"+id + "_clear").addClass("hidden");';
			$script[] = '		if (document.getElementById(id + "_edit")) {';
			$script[] = '			jQuery("#"+id + "_edit").addClass("hidden");';
			$script[] = '		}';
			$script[] = '		return false;';
			$script[] = '	}';
		}

		// Add the script to the document head.
		JFactory::getDocument()->addScriptDeclaration(implode("\n", $script));

		// Setup variables for display.
		$html	= array();
		$link	= 'index.php?option=com_amazons3&amp;view=objects&amp;layout=modal&amp;tmpl=component&amp;function=jSelectObject_'.$this->id;

		if (isset($this->element['language']))
		{
			$link .= '&amp;forcedLanguage='.$this->element['language'];
		}

        $title = "";
		$valueObject = "";
		$valueBucket = "";
		
		// Get the title of the linked chart
		if (!empty($this->value["object"]))
		{
			$title = $this->value["object"];
			$valueObject = $this->value["object"];			
		}
		if (!empty($this->value["bucket"]))
		{
			$valueBucket = $this->value["bucket"];			
		}

		if (empty($title))
		{
			$title = JText::_('COM_AMAZONS3_SELECT_AN_OBJECT');
		}
		$title = htmlspecialchars($title, ENT_QUOTES, 'UTF-8');


		// The current object display field.
		$html[] = '<span class="input-append">';
		$html[] = '<input type="text" class="input-medium" id="'.$this->id.'_bucket" name="'.$this->name.'[bucket]" value="'.$valueBucket.'" readonly="readonly" size="35"  />';		
		$html[] = '<input type="text" class="input-large" id="'.$this->id.'_object" name="'.$this->name.'[object]" value="'.$valueObject.'" readonly="readonly" size="35" />';
		$html[] = '<a class="modal btn hasTooltip" title="'.JHtml::tooltipText('COM_AMAZONS3_CHANGE_OBJECT_BUTTON').'"  href="'.$link.'&amp;'.JSession::getFormToken().'=1" rel="{handler: \'iframe\', size: {x: 800, y: 450}}"><i class="icon-file"></i> '.JText::_('JSELECT').'</a>';

		// Edit object button
		if ($allowEdit)
		{
			$html[] = '<a class="btn hasTooltip'.($valueObject ? '' : ' hidden').'" href="index.php?option=com_amazons3&layout=modal&tmpl=component&task=object.edit&id=' . $value. '" target="_blank" title="'.JHtml::tooltipText('COM_AMAZONS3_EDIT_OBJECT').'" ><span class="icon-edit"></span> ' . JText::_('JACTION_EDIT') . '</a>';
		}

		// Clear package button
		if ($allowClear)
		{
			$html[] = '<button id="'.$this->id.'_clear" class="btn'.($valueObject ? '' : ' hidden').'" onclick="return jClearObject(\''.$this->id.'\')"><span class="icon-remove"></span> ' . JText::_('JCLEAR') . '</button>';
		}

		$html[] = '</span>';

		// class='required' for client side validation
		$class = '';

		if ($this->required)
		{
			$class = ' class="required modal-value"';
		}


		return implode("\n", $html);
	}
}
