CREATE TABLE IF NOT EXISTS `#__amazons3_config` (
  `id` int(11) NOT NULL,
  `secretkey` blob,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 DEFAULT COLLATE=utf8mb4_unicode_ci;

INSERT IGNORE INTO `#__amazons3_config` (`id`, `secretkey`) VALUES (1, NULL); 

CREATE TABLE IF NOT EXISTS `#__amazons3_packages` (
	`id` int(11) unsigned NOT NULL AUTO_INCREMENT,
	`asset_id` int(11) unsigned NOT NULL DEFAULT '0' COMMENT 'FK to the #__assets table.',
	`catid` int(11) unsigned NOT NULL DEFAULT '0',
	`code` VARCHAR(255) NOT NULL COMMENT 'A randomly generated unique code to identify the package',
	`isfree` TINYINT(3) NOT NULL COMMENT 'If this is set, then files in the package are freely downloadable, otherwise not.',
	`title` VARCHAR(255) NOT NULL,
	`description` text NOT NULL COMMENT 'description', 
	`ecommerce_name` VARCHAR(255) NOT NULL COMMENT 'the name of the ecommerce extension through which the package will be sold as a product',
	`eco_product_id` int(11) unsigned NOT NULL DEFAULT '0' COMMENT 'the product id in the ecommerce extension products table',
	`price` decimal(12,4) NOT NULL DEFAULT '0.0' COMMENT 'the price of the package',
	`alias` varchar(255) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL DEFAULT '',	
	`ordering` int(11) NOT NULL DEFAULT '0',
	`published` tinyint(3) NOT NULL DEFAULT '0',
	`checked_out` int(11) unsigned NOT NULL DEFAULT '0',
	`checked_out_time` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`created` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`created_by` int(11) unsigned NOT NULL DEFAULT '0',
	`created_by_alias` varchar(255) NOT NULL DEFAULT '',
	`modified` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`modified_by` int(11) unsigned NOT NULL DEFAULT '0',
	`publish_up` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`publish_down` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`images` text NOT NULL,
	`version` int(11) unsigned NOT NULL DEFAULT '1',
	`hits` int(11) NOT NULL DEFAULT '0',
	`access` int(11) unsigned NOT NULL DEFAULT '0',
	`language` char(7) NOT NULL COMMENT 'The language code for the package.',
	`params` text NOT NULL COMMENT 'includes parameters such as the number of days and number of times can be downloaded',
	`metadata` text NOT NULL,
	`metakey` text NOT NULL,
	`metadesc` text NOT NULL,
	 PRIMARY KEY (id),
	 KEY `idx_code` (`code`),
	 KEY `idx_product` (`eco_product_id`)
)
ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 DEFAULT COLLATE=utf8mb4_unicode_ci;


CREATE TABLE IF NOT EXISTS `#__amazons3_packages_files` (
	`id` int(11) unsigned NOT NULL AUTO_INCREMENT,
	`asset_id` int(11) unsigned NOT NULL DEFAULT '0' COMMENT 'FK to the #__assets table.',
	`package` int(10) unsigned NOT NULL DEFAULT '0',
	`code` VARCHAR(255) NOT NULL COMMENT 'Randomly generated unique code for the file',
	`isfree` TINYINT(3) NOT NULL COMMENT 'If this is set, then file is freely downloadable, otherwise not.',	
	`file_version` VARCHAR(255) NOT NULL COMMENT 'Use to store the file version number',
	`source` VARCHAR(255) NOT NULL COMMENT 'indicate where file is hosted, usually this will be AmazonS3',
	`title` VARCHAR(255) NOT NULL,
	`description` text NOT NULL COMMENT 'description', 
	`alias` varchar(255) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL DEFAULT '',	
	`ordering` int(11) NOT NULL DEFAULT '0',
	`published` tinyint(3) NOT NULL DEFAULT '0',
	`checked_out` int(11) unsigned NOT NULL DEFAULT '0',
	`checked_out_time` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`created` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`created_by` int(11) unsigned NOT NULL DEFAULT '0',
	`created_by_alias` varchar(255) NOT NULL DEFAULT '',
	`modified` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`modified_by` int(11) unsigned NOT NULL DEFAULT '0',
	`publish_up` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`publish_down` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`images` text NOT NULL,
	`version` int(11) unsigned NOT NULL DEFAULT '1',
	`hits` int(11) NOT NULL DEFAULT '0',
	`access` int(11) unsigned NOT NULL DEFAULT '0',
	`language` char(7) NOT NULL COMMENT 'The language code for the file.',
	`params` text NOT NULL,
	`metadata` text NOT NULL,
	`metakey` text NOT NULL,
	`metadesc` text NOT NULL,
	 PRIMARY KEY (id),
	 KEY `idx_code` (`code`)	
)
ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 DEFAULT COLLATE=utf8mb4_unicode_ci;


CREATE TABLE IF NOT EXISTS `#__amazons3_orders` (
	`id` int(11) unsigned NOT NULL AUTO_INCREMENT,
	`ecommerce_name` VARCHAR(255) NOT NULL COMMENT 'the name of the ecommerce extension through which the order was placed',
	`eco_order_id` int(11) unsigned NOT NULL DEFAULT '0' COMMENT 'the order id in the ecommerce extension orders table',
	`code` VARCHAR(255) NOT NULL COMMENT 'Randomly generated unique code for the order',
	`user_id` int(11) unsigned NOT NULL DEFAULT '0' COMMENT 'the user id in the joomla users table',
	`user_name` VARCHAR(255) NOT NULL COMMENT 'user name, useful if no user account is created',	
	`user_email` VARCHAR(255) NOT NULL COMMENT 'user email address, useful if no user account is created',
	`user_data` text NOT NULL,	
	`published` tinyint(3) NOT NULL DEFAULT '0',
	`publish_up` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`publish_down` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`created` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`created_by` int(11) NOT NULL DEFAULT '0',
	`modified_on` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`modified_by` int(11) NOT NULL DEFAULT '0',
	`locked_on` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`locked_by` int(11) NOT NULL DEFAULT '0',	
	 PRIMARY KEY (id),
	 KEY `idx_code` (`code`),
	 KEY `idx_eco_order_id` (`eco_order_id`)	 
)
ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 DEFAULT COLLATE=utf8mb4_unicode_ci;

CREATE TABLE IF NOT EXISTS `#__amazons3_orders_items` (
	`id` int(11) unsigned NOT NULL AUTO_INCREMENT,
	`order_id` int(11) unsigned NOT NULL DEFAULT '0' COMMENT 'the order id in the amazons3 orders table',
	`code` VARCHAR(255) NOT NULL COMMENT 'Randomly generated unique code for the order item',
	`eco_product_id` int(11) unsigned NOT NULL DEFAULT '0' COMMENT 'the product id in the ecommerce extension products table',
	`published` tinyint(3) NOT NULL DEFAULT '0',		
	`package` int(10) unsigned NOT NULL DEFAULT '0' COMMENT 'the package id of this item',	
	`hits` int(11) unsigned NOT NULL DEFAULT '0' COMMENT 'the number of downloads for this item',
	`downloads_end` datetime NOT NULL DEFAULT '0000-00-00 00:00:00' COMMENT 'when downloads expire for this item',
	`downloads_limit` int(11) unsigned NOT NULL DEFAULT '0' COMMENT 'the total number of downloads available for this package',
	`created` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`created_by` int(11) NOT NULL DEFAULT '0',
	`modified_on` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`modified_by` int(11) NOT NULL DEFAULT '0',
	`locked_on` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
	`locked_by` int(11) NOT NULL DEFAULT '0',
	 PRIMARY KEY (id),
	 KEY `idx_code` (`code`),
	 KEY `idx_order_id` (`order_id`)	 
	 	 
)
ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 DEFAULT COLLATE=utf8mb4_unicode_ci;