<?php
/**
 * @author Fiona Coulter		
 * @copyright Spiral Scripts
 * @license GPL		
 */

defined("_JEXEC") or die("Restricted access");


$s3_path = JPATH_COMPONENT.'/helpers/';

JLoader::register('AmazonS3APIHelper',$s3_path.'amazons3api.php');
JLoader::register('AmazonS3SiteHelper',$s3_path.'amazons3.php');
JLoader::register('DropboxHelper',$s3_path.'dropbox.php');
JLoader::register('DropboxapiHelper',$s3_path.'dropboxapi.php');


/**
 * Files list controller class.
 *
 * @package     Amazons3
 * @subpackage  Controllers
 */
class Amazons3ControllerFile extends JControllerLegacy
{
	/**
	 * The URL view list variable.
	 *
	 * @var    string
	 * @since  12.2
	 */
	protected $view_list = 'files';
    public function display($cachable = false, $urlparams = false)
	{
		
		parent::display(true, false);
    }
	
	public function download()
	{
		$app = JFactory::getApplication();
		$code = $app->input->get("code","","string");
		if(empty($code))
		{
			JError::raiseError(404, JText::_('AMAZONS3_FILENOTFOUND'));
			return;
		}
		
		
		
		$model = $this->getModel("file");
		$item = $model->getItem();
		if(!$item)
		{
			JError::raiseError(404, JText::_('AMAZONS3_FILENOTFOUND'));
			return;
		}
		if(!$item->isfree)
		{
			JError::raiseWarning(403, JText::_('JERROR_ALERTNOAUTHOR'));
			return;
		}
		if(isset($item->package_id) && $item->package_id > 0 && !$item->package_isfree)
		{
			JError::raiseWarning(403, JText::_('JERROR_ALERTNOAUTHOR'));
			return;
		}		
		if($item->params->get('access-view') == false)
		{
			JError::raiseWarning(403, JText::_('JERROR_ALERTNOAUTHOR'));
			return;
		}
		$model->hit($item->id);
		if(isset($item->package_id) && $item->package_id > 0)
		{
			$packageModel = $this->getModel("package");
			$packageModel->hit($item->package_id);
		}
		
		
		$source = $item->source;
		
		switch($source)
		{
			case "amazons3":
			
			  $params = AmazonS3SiteHelper::getParams();
			  $secretkey = $params->get("secret_key","");
			  $publickey = $params->get("access_key","");
			  
			  if(empty($secretkey) || empty($publickey))
			  {
				  $errorMessage = JText::_('AS3_SECRETKEYNOTSET');
				  //JError::raiseWarning(100, $errorMessage);
				  $app->enqueueMessage($errorMessage,'error');
				  $app->redirect(JRoute::_('index.php'));
				  return;
			  }
			  
			  $params->merge($item->params);
			

			  $location = $item->params->get("location");
			  if(isset($location["bucket"]) && isset($location["object"]) && !empty($location["bucket"]) && !empty($location["object"]))
			  {
				  if(!AmazonS3APIHelper::download($params, $location["bucket"], $location["object"]))
				  {
					  $msg = AmazonS3APIHelper::getErrorMessage();
					  $app->enqueueMessage($errorMessage,'error');
					  $app->redirect(JRoute::_('index.php'));
					  return;
					  
				  }
				  else
				  {
					  jexit();
	  
				  }
			  }
			  break;
			  
		   case 'dropbox':
		   			  $params = AmazonS3SiteHelper::getParams(DROPBOX_KEY_ID);
					  $token = DropboxHelper::getToken($params);
					  if(empty($token))
					  {
						  $errorMessage = JText::_('ASC_DROPBOX_TOKEN_NOT_SET');
						  $app->enqueueMessage($errorMessage,'error');
						  $app->redirect(JRoute::_('index.php'));
						  return;
					  }
					  
					  $params->merge($item->params);
					  $dlocation = $item->params->get("dlocation");
					  
					  if(empty($dlocation))
					  {
							$errorMessage = JText::_('AS3_DROPBOX_FILE_PATH_EMPTY');
							$app->enqueueMessage($errorMessage,'error');
							$app->redirect(JRoute::_('index.php'));
							return;
						  
						  
					  }
					  $options = json_decode($dlocation, true);
					  $options["disposition"] = $params->get("downloadType","attachment");
					  
					  $downloadType = $item->params->get("dropbox_download_type", "standard");
					  //$model->hit($item->id);
					  
					  
					  if($downloadType == "standard")
					  {
						  DropboxapiHelper::filesDownload($token, $options);
						  jexit();
						  
					  }
					  elseif($downloadType == "redirect")
					  {
						  $link = DropboxapiHelper::filesGetTemporaryLink($token, $options);
						  $app->redirect($link);
						  return;
						  
					  }
					  

		      break;
			  default: break;
			
		}
		
		
		//for some reason we have reached here
		JError::raiseError(404, JText::_('AMAZONS3_FILENOTFOUND'));
		return;
		
		

	}
	
	
}
?>