<?php
/**
 * @author Fiona Coulter		
 * @copyright Spiral Scripts
 * @license GPL		
 */

defined("_JEXEC") or die("Restricted access");

use Joomla\Registry\Registry;
/**
 * Item Model for file.
 *
 * @package     Amazons3
 * @subpackage  Models
 */
class Amazons3ModelFile extends JModelItem
{
	/**
	 * @var        string    The prefix to use with controller messages.
	 * @since   1.6
	 */
	protected $text_prefix = 'COM_AMAZONS3';

	/**
	 * The type alias for this content type.
	 *
	 * @var      string
	 * @since    3.2
	 */
	public $typeAlias = 'com_amazons3.file';
	protected $_context = 'com_amazons3.file';	

	/**
	 * Method to test whether a record can be deleted.
	 *
	 * @param   object    $record    A record object.
	 *
	 * @return  boolean  True if allowed to delete the record. Defaults to the permission set in the component.
	 * @since   1.6
	 */
	protected function canDelete($record)
	{
		if (!empty($record->id))
		{
			if ($record->published != -2)
			{
				return false;
			}
			

			$user = JFactory::getUser();
			return $user->authorise('core.delete', $this->typeAlias . '.' . (int) $record->id);
		}
	}

	/**
	 * Method to test whether a record can have its state edited.
	 *
	 * @param   object    $record    A record object.
	 *
	 * @return  boolean  True if allowed to change the state of the record. Defaults to the permission set in the component.
	 * @since   1.6
	 */
	protected function canEditState($record)
	{
		$user = JFactory::getUser();

		// Check for existing item.
		if (!empty($record->id))
		{
			return $user->authorise('core.edit.state', $this->typeAlias . '.' . (int) $record->id);
		}
		// Default to component settings if neither article nor category known.
		else
		{
			return parent::canEditState($this->option);
		}
	}
		

	/**
	 * Prepare and sanitise the table data prior to saving.
	 *
	 * @param   JTable    A JTable object.
	 *
	 * @return  void
	 * @since   1.6
	 */
	protected function prepareTable($table)
	{
		// Set the publish date to now
		$db = $this->getDbo();
		
		if ($table->published == 1 && (int) $table->publish_up == 0)
		{
			$table->publish_up = JFactory::getDate()->toSql();
		}

		if ($table->published == 1 && intval($table->publish_down) == 0)
		{
			$table->publish_down = $db->getNullDate();
		}

		// Increment the content version number.
		//$table->version++;
	}

	/**
	 * Auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 *
	 * @return  void
	 *
	 * @since   1.6
	 */
	protected function populateState()
	{
		$app = JFactory::getApplication();

		// Load the User state.
		$pk = $app->input->getInt('id',0);
		$this->setState('file.id', $pk);
		
		$code = $app->input->get("code","","string");
		$this->setState("file.code",$code);
		
		$this->setState('filter.published', 1);
		//$this->setState('filter.archived', 2);
		
		$this->setState('filter.language', JLanguageMultilang::isEnabled());

		// Load the parameters.
		$params = $app->getParams();
		$this->setState('params', $params);
	}
	
	/**
	 * Alias for JTable::getInstance()
	 *
	 * @param   string  $type    The type (name) of the JTable class to get an instance of.
	 * @param   string  $prefix  An optional prefix for the table class name.
	 * @param   array   $config  An optional array of configuration values for the JTable object.
	 *
	 * @return  mixed    A JTable object if found or boolean false if one could not be found.
	 */
	public function getTable($type = 'File', $prefix = 'Amazons3Table', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}
	
	
	/**
	 * Method to get a single record.
	 *
	 * @param	integer	The id of the primary key.
	 *
	 * @return	mixed	Object on success, false on failure.
	 * @since	1.6
	 */
	 
	public function getItem($pk = null)
	{
		$user = JFactory::getUser();

		$pk = (!empty($pk)) ? $pk : (int) $this->getState('file.id',0);
		$code = $this->getState("file.code","");
		if(empty($pk) && empty($code))
		{
			JError::raiseError(404, JText::_('COM_AMAZONS3_ERROR_FEED_NOT_FOUND'));
		}
		

		if ($this->_item === null)
		{
			$this->_item = array();
		}
		
		if (!isset($this->_item[$pk])||empty($pk))
		{
			try
			{
				$db = $this->getDbo();
				$query = $db->getQuery(true)
					->select(
						'a.id, a.asset_id, a.title, a.source, b.title as package_title, a.alias, a.description, a.code, b.code as package_code, b.id as package_id, '.
						 'a.file_version, a.isfree, b.isfree as package_isfree, b.access as package_access, b.published as package_published, ' .
							// If badcats is not null, this means that the file is inside an unpublished category or package
							// In this case, the published is set to 0 to indicate Unpublished (even if the file state is Published)
							'CASE WHEN badcats.id is null THEN a.published ELSE 0 END AS published, ' .
							'b.catid, a.created, a.created_by, a.created_by_alias, ' .
							// Use created if modified is 0
							'CASE WHEN a.modified = ' . $db->quote($db->getNullDate()) . ' THEN a.created ELSE a.modified END as modified, ' .
							'a.modified_by, a.checked_out, a.checked_out_time, a.publish_up, a.publish_down, ' .
							'a.images, a.params, a.version, a.ordering, ' .
							'a.metakey, a.metadesc, a.access, a.hits, a.metadata, a.language'
					);
				$query->from('#__amazons3_packages_files AS a');
				
				$query->join('LEFT','#__amazons3_packages AS b on a.package = b.id');
				
				// Join on category table.
				$query->select('c.title AS category_title, c.alias AS category_alias, c.access AS category_access')
					->join('LEFT', '#__categories AS c on c.id = b.catid');
					
				// Join on user table.
				$query->select('u.name AS author')
					->join('LEFT', '#__users AS u on u.id = a.created_by');

				// Filter by language
				if ($this->getState('filter.language'))
				{
					$query->where('a.language in (' . $db->quote(JFactory::getLanguage()->getTag()) . ',' . $db->quote('*') . ')');
				}

				// Join over the categories to get parent category titles
				$query->select('parent.title as parent_title, parent.id as parent_id, parent.path as parent_route, parent.alias as parent_alias')
					->join('LEFT', '#__categories as parent ON parent.id = c.parent_id');

				// Filter by start and end dates.
				$nullDate = $db->quote($db->getNullDate());
				$date = JFactory::getDate();

				$nowDate = $db->quote($date->toSql());

				$query->where('(a.publish_up = ' . $nullDate . ' OR a.publish_up <= ' . $nowDate . ')')
					->where('(a.publish_down = ' . $nullDate . ' OR a.publish_down >= ' . $nowDate . ')');

				// Join to check for category published state in parent categories up the tree
				// If all categories are published, badcats.id will be null, and we just use the article state
				$subquery = ' (SELECT cat.id as id FROM #__categories AS cat JOIN #__categories AS parent ';
				$subquery .= 'ON cat.lft BETWEEN parent.lft AND parent.rgt ';
				$subquery .= 'WHERE parent.extension = ' . $db->quote('com_amazons3');
				$subquery .= ' AND parent.published <= 0 GROUP BY cat.id)';
				$query->join('LEFT OUTER', $subquery . ' AS badcats ON badcats.id = c.id');
				

				$query->where('(a.isfree = 1 AND (b.id is null OR b.isfree = 1))');

				$query->where('(a.published = 1 AND (b.id is null OR b.published = 1))');
				$query->where('(b.id is null OR b.publish_up = ' . $nullDate . ' OR b.publish_up <= ' . $nowDate . ')')
					->where('(b.id is null OR b.publish_down = ' . $nullDate . ' OR b.publish_down >= ' . $nowDate . ')');
				
				$code = $this->getState('file.code');
				if(!empty($code))
				{
					$query->where('(a.code = '.$db->quote($code).')');
				}
				
				if(!empty($pk))
				{
					$query->where('(a.id = '.(int)$pk.')');
				}
				

				$db->setQuery($query);

				$data = $db->loadObject();

				if (empty($data))
				{
					//return JError::raiseError(404, JText::_('COM_AMAZONS3_FILE_NOT_FOUND'));
					return false;
				}

				// Check for published state if filter set.
				if ($data->published != 1)
				{
					//return JError::raiseError(404, JText::_('COM_AMAZONS3_FILE_NOT_FOUND'));
					return false;
				}
				
				// Convert parameter fields to objects.
				$registry = new Registry;
				$registry->loadString($data->params);

				$data->params = clone $this->getState('params');
				$data->params->merge($registry);

				$registry = new Registry;
				$registry->loadString($data->metadata);
				$data->metadata = $registry;
				
				$registry = new Registry;
				$registry->loadString($data->images);
				$data->images = $registry;
				
				
				$user = JFactory::getUser();
				$groups = $user->getAuthorisedViewLevels();

				if ($data->catid == 0 || $data->category_access === null)
				{
					if($data->package_id == 0 || $data->package_access === null)
					{
						$data->params->set('access-view', in_array($data->access, $groups));
						$data->params->set('package-access-view', true);
					}
					else
					{
						$data->params->set('access-view', in_array($data->access, $groups) && in_array($data->package_access, $groups));
						$data->params->set('package-access-view', in_array($data->package_access, $groups));
					}
				}
				else
				{
					if($data->package_id == 0 || $data->package_access === null)
					{
						  $data->params->set('access-view', in_array($data->access, $groups) && in_array($data->category_access, $groups));
						  $data->params->set('package-access-view', true);

					}
					else
					{
						  $data->params->set('access-view', in_array($data->access, $groups) && in_array($data->category_access, $groups) && in_array($data->package_access, $groups));	
						  $data->params->set('package-access-view', in_array($data->package_access, $groups) && in_array($data->category_access, $groups));
						  
					}
				}
				
				$this->_item[$pk] = $data;

				
			}
			catch (Exception $e)
			{
				if ($e->getCode() == 404)
				{
					// Need to go thru the error handler to allow Redirect to work.
					//JError::raiseError(404, $e->getMessage());
					$this->setError($e);
					$this->_item[$pk] = false;
					
				}
				else
				{
					$this->setError($e);
					$this->_item[$pk] = false;
				}
			}

		}//end if !isset
		
		return $this->_item[$pk];
		
	}
	
	public function getFileForOrder($package, $code)
	{
		//gets the file by code and package
		if(empty($code))
		{
			return false;
		}
		
		$db = $this->getDbo();
		$query = $db->getQuery(true)
			->select("a.*")->from("#__amazons3_packages_files as a");
			
		if(!empty($code))
		{
			$query->where('(a.code = '.$db->quote($code).')');
		}
		
		if(!empty($package))
		{
			$query->where('(a.package = '.$db->quote($package).')');
		}
		
		$query->where('(a.published = 1)');
		
		
		// Filter by start and end dates.
		$nullDate = $db->quote($db->getNullDate());
		$date = JFactory::getDate();

		$nowDate = $db->quote($date->toSql());

		$query->where('(a.publish_up = ' . $nullDate . ' OR a.publish_up <= ' . $nowDate . ')')
			->where('(a.publish_down = ' . $nullDate . ' OR a.publish_down >= ' . $nowDate . ')');
		
		$db->setQuery($query);

		$data = $db->loadObject();

		if (empty($data))
		{
			return false;
		}

		// Check for published state if filter set.
		if ($data->published != 1)
		{
			return false;
		}
		
		// Convert parameter fields to objects.
		$registry = new Registry;
		$registry->loadString($data->params);

		$data->params = clone $this->getState('params');
		$data->params->merge($registry);	
		
		return $data;

		
	}
	 
	
	/**
	 * Increment the hit counter for the item.
	 *
	 * @param   integer  $pk  Optional primary key of the item to increment.
	 *
	 * @return  boolean  True if successful; false otherwise and internal error set.
	 */
	public function hit($pk = 0)
	{
		$input = JFactory::getApplication()->input;
		$hitcount = $input->getInt('hitcount', 1);

		if ($hitcount)
		{
			$pk = (!empty($pk)) ? $pk : (int) $this->getState('file.id');

			$table = $this->getTable();
			$table->load($pk);
			$table->hit($pk);
		}

		return true;
	}
}
?>