<?php
/**
 * @author Fiona Coulter		
 * @copyright Spiral Scripts
 * @license GPL		
 */

defined("_JEXEC") or die("Restricted access");

use Joomla\Registry\Registry;

/**
 * List Model for files.
 *
 * @package     Amazons3
 * @subpackage  Models
 */
class Amazons3ModelFiles extends JModelList
{
	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 */
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'a.title', 'title',
				'a.alias', 'alias',
				'a.created', 'created',
				'a.ordering', 'ordering',
				'a.language', 'language',
				'a.hits', 'hits',
				'a.package', 'package'
			);
		}
		parent::__construct($config);
	}
	
	/**
	 * Method to auto-populate the model state.
	 *
	 * This method should only be called once per instantiation and is designed
	 * to be called on the first call to the getState() method unless the model
	 * configuration flag to ignore the request is set.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 *
	 * @param   string  $ordering   An optional ordering field.
	 * @param   string  $direction  An optional direction (asc|desc).
	 *
	 * @return  void
	 */
	protected function populateState($ordering = 'ordering', $direction = 'ASC')
	{
		// Get the Application
		$app = JFactory::getApplication();
		
		
		$package = $app->input->get("package","");
		if(empty(!$package))
		{
			$this->setState('filter.package', $package);
		}
		
		// Receive & set filters
		if ($filters = $app->getUserStateFromRequest($this->context . '.filter', 'filter', array(), 'array'))
		{
			foreach ($filters as $name => $value)
			{
				
				if (in_array($name, $this->filter_fields))
				{
					$this->setState('filter.' . $name, $value);
				}
			}
		}
		
		// List state information
		$value = $app->getUserStateFromRequest($this->context . '.limit', 'limit', $app->get('list_limit', 0), 'uint');
		$this->setState('list.limit', $value);
		$value = $app->getUserStateFromRequest($this->context . '.limitstart', 'limitstart', 0, 'uint');		
		$this->setState('list.start', $value);

		
		// Set filter state for search
        $search = $app->getUserStateFromRequest($this->context . '.filter.search', 'filter_search');
        $this->setState('filter.search', $search);

		$this->setState('filter.access', 1);

		// Set filter state for author
		$authorId = $app->getUserStateFromRequest($this->context . '.filter.author_id', 'filter_author_id');
		$this->setState('filter.author_id', $authorId);

        $this->setState('filter.published', 1);

		// Set filter state for language
		$language = $this->getUserStateFromRequest($this->context . '.filter.language', 'filter_language', '');
		$this->setState('filter.language', $language);		// Set filter state for package
		//$package = $this->getUserStateFromRequest($this->context.'.filter.package', 'filter_package', '');
		
		
		$isadmin = 0;
		$user = JFactory::getUser();
		if ($user->authorise('core.admin'))
		{
			$isadmin = 1;
		}
        $this->setState('filter.isadmin', $isadmin);
		
		
		
		// force a language
		$forcedLanguage = $app->input->get('forcedLanguage');

		// Set filter state for language if a language is forced
		if (!empty($forcedLanguage))
		{
			$this->setState('filter.language', $forcedLanguage);
			$this->setState('filter.forcedLanguage', $forcedLanguage);
		}
		
		$list = $app->getUserStateFromRequest($this->context . '.list', 'list', array(), 'array');
		if(isset($list["fullordering"]))
		{
			$listordering = $list["fullordering"];	
		}
		else
		{
			$listordering = "";
		}
		$ordArray = explode(" ",$listordering);
	
		if(isset($ordArray[0]))
		{
			$value = $ordArray[0];
			if(in_array($value, $this->filter_fields))
			{
				$this->setState("list.ordering", $value);
			}
			else
			{
				$this->setState("list.ordering", "a.ordering");
			}
		}
		else
		{
			$this->setState("list.ordering","a.ordering");
		}
		
		if(isset($ordArray[1]))
		{
			$value = $ordArray[1];
			if (in_array(strtoupper($value), array('ASC', 'DESC')))
			{
				$this->setState("list.direction", $ordArray[1]);
			}
			else
			{
				$this->setState("list.direction", 'ASC');
			}
		}
		else
		{
			$this->setState("list.direction","ASC");
		}		

		// Load the parameters.
		$params = $app->getParams();
		$this->setState('params', $params);
		
		$no_access_files = $params->get("no_access_files","show");
		$this->setState("no_access_files", $no_access_files);

		// List state information.
		//parent::populateState($ordering, $direction);
	}
	
	/**
	 * Method to get a store id based on model configuration state.
	 *
	 * This is necessary because the model is used by the component and
	 * different modules that might need different sets of data or different
	 * ordering requirements.
	 *
	 * @param   string  $id  A prefix for the store id.
	 *
	 * @return  string  A store id.
	 *
	 * @since   1.6
	 */
	protected function getStoreId($id = '')
	{
		// Compile the store id.
		$id .= ':' . $this->getState('filter.search');
		$id .= ':' . $this->getState('filter.isadmin');
		$id .= ':' . $this->getState('filter.published');
		$id .= ':' . $this->getState('filter.author_id');
		$id .= ':' . $this->getState('filter.language');
		$id .= ':' . $this->getState('filter.package');

		return parent::getStoreId($id);
	}

	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return  JDatabaseQuery
	 */
	protected function getListQuery()
	{
		// Get database object
		$db = $this->getDbo();
		$query = $db->getQuery(true);
		$query->select('a.*')->from('#__amazons3_packages_files AS a');		//Join over packages
		$query->select('amazons3_packages.title AS package_title, amazons3_packages.access as package_access')
			->join('LEFT', '#__amazons3_packages AS amazons3_packages ON amazons3_packages.id = a.package');

		$query->select('c.title AS category_title, c.id as catid, c.alias AS category_alias, c.access AS category_access')
		    ->join('LEFT', '#__categories AS c ON c.id = amazons3_packages.catid');
			
		// Join over the asset groups.
		$query->select('ag.title AS access_level')
			->join('LEFT', '#__viewlevels AS ag ON ag.id = a.access');
			

		
		// Join over the language
		$query->select('l.title AS language_title')
			->join('LEFT', $db->quoteName('#__languages') . ' AS l ON l.lang_code = a.language');
			
		// Join to check for category published state in parent categories up the tree
		// If all categories are published, badcats.id will be null, and we just use the article state
		$subquery = ' (SELECT cat.id as id FROM #__categories AS cat JOIN #__categories AS parent ';
		$subquery .= 'ON cat.lft BETWEEN parent.lft AND parent.rgt ';
		$subquery .= 'WHERE parent.extension = ' . $db->quote('com_amazons3');
		$subquery .= ' AND parent.published <= 0 GROUP BY cat.id)';
		$query->join('LEFT OUTER', $subquery . ' AS badcats ON badcats.id = c.id');
			
		
				
		// Join over the users for the author.
		$query->select('ua.name AS author_name')
			->join('LEFT', '#__users AS ua ON ua.id = a.created_by');

		// Join over the users for the modifier.
		$query->select('um.name AS modifier_name')
			->join('LEFT', '#__users AS um ON um.id = a.modified_by');

		// Filter by search
		$search = $this->getState('filter.search');
		
		if (!empty($search))
		{
			if (stripos($search, 'id:') === 0)
			{
				$query->where('a.id = ' . (int) substr($search, strlen('id:')));
			}
			elseif (stripos($search, 'title:') === 0)
			{
				$search = $db->quote('%' . substr($search, strlen('title:')) . '%');
				$query->where('(a.title LIKE ' . $search);
			}
			elseif (stripos($search, 'author:') === 0)
			{
				$search = $db->quote('%' . substr($search, 7) . '%');
				$query->where('(ua.name LIKE ' . $search . ' OR ua.username LIKE ' . $search . ')');
			}
			else
			{
				$search = $db->quote('%' . $search . '%');
				$query->where('(a.title LIKE ' . $search.' OR a.description LIKE ' . $search .')');
				
			}
		}
		
		// Filter by published state.
		$query->where('(a.isfree = 1 AND (amazons3_packages.id is null OR amazons3_packages.isfree = 1))');
		// Filter by start and end dates.
		$nullDate = $db->quote($db->getNullDate());
		$date = JFactory::getDate();
		$nowDate = $db->quote($date->toSql());
		

		$query->where('(a.published = 1 AND (amazons3_packages.id is null OR amazons3_packages.published = 1))');
		$query->where('(a.publish_up = ' . $nullDate . ' OR a.publish_up <= ' . $nowDate . ')')
			->where('(a.publish_down = ' . $nullDate . ' OR a.publish_down >= ' . $nowDate . ')');
		
		
		$query->where('(amazons3_packages.id is null OR amazons3_packages.publish_up = ' . $nullDate . ' OR amazons3_packages.publish_up <= ' . $nowDate . ')')
			->where('(amazons3_packages.id is null OR amazons3_packages.publish_down = ' . $nullDate . ' OR amazons3_packages.publish_down >= ' . $nowDate . ')');

        $query->where('badcats.id is null');
		
		// Filter on the language.
		if ($language = $this->getState('filter.language'))
		{
			$query->where('a.language = ' . $db->quote($language));
		}
		
		// Filter by author
		$authorId = $this->getState('filter.author_id');
		if (is_numeric($authorId))
		{
			$type = $this->getState('filter.author_id.include', true) ? '= ' : '<>';
			$query->where('a.created_by ' . $type . (int) $authorId);
		}
		
		
		// Implement View Level Access
		$user = JFactory::getUser();
		$no_access_files = $this->getState("no_access_files");
		if (!$user->authorise('core.admin'))
		{
			$groups = implode(',', $user->getAuthorisedViewLevels());
			if($no_access_files == "hide")
			{
				$query->where('a.access IN (' . $groups . ')');
			}
			$query->where('(amazons3_packages.id is null OR amazons3_packages.access IN (' . $groups . '))');
			$query->where('(c.id is null OR c.access IN (' . $groups . '))');
			
		}		// Filter by package
		$package = $this->getState('filter.package');
		if ($package != "")
		{
			$query->where('a.package = ' . $db->quote($package));
		}


		
		// Add list oredring and list direction to SQL query
		$sort = $this->getState('list.ordering', 'ordering');
		$order = $this->getState('list.direction', 'ASC');
		$query->order($db->escape($sort).' '.$db->escape($order));
		
		return $query;
	}
	
	/**
	 * Build a list of authors
	 *
	 * @return  array
	 *
	 * @since   1.6
	 */
	public function getAuthors()
	{
		// Create a new query object.
		$db = $this->getDbo();
		$query = $db->getQuery(true);

		// Construct the query
		$query->select('u.id AS value, u.name AS text')
			->from('#__users AS u')
			->join('INNER', '#__amazons3_packages_files AS a ON a.created_by = u.id')
			->group('u.id, u.name')
			->order('u.name');

		// Setup the query
		$db->setQuery($query);

		// Return the result
		return $db->loadObjectList();
	}
	
	/**
	 * Method to get an array of data items.
	 *
	 * @return  mixed  An array of data items on success, false on failure.
	 *
	 * @since   12.2
	 */
	public function getItems()
	{
		$user = JFactory::getUser();
		$groups = $user->getAuthorisedViewLevels();
		//print_r($groups);
		
		
		if ($items = parent::getItems()) {
			foreach($items as $item)
			{
				$registry = new Registry;
				$registry->loadString($item->params);
				$item->params = $registry;
				$item->params->set('access-view', in_array($item->access, $groups));
				
				$registry = new Registry;
				$registry->loadString($item->images);
				$item->images = $registry;
				
			}
		}

		return $items;
	}
}
?>