<?php
/**
 * @package     Joomla.Site
 * @subpackage  Templates.protostar
 *
 * @copyright   Copyright (C) 2005 - 2019 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

/** @var JDocumentHtml $this */

$app  = JFactory::getApplication();
$user = JFactory::getUser();

// Output as HTML5
$this->setHtml5(true);

// Getting params from template
$params = $app->getTemplate(true)->params;

// Detecting Active Variables
$option   = $app->input->getCmd('option', '');
$view     = $app->input->getCmd('view', '');
$layout   = $app->input->getCmd('layout', '');
$task     = $app->input->getCmd('task', '');
$itemid   = $app->input->getCmd('Itemid', '');
$sitename = htmlspecialchars($app->get('sitename'), ENT_QUOTES, 'UTF-8');

if ($task === 'edit' || $layout === 'form')
{
	$fullWidth = 1;
}
else
{
	$fullWidth = 0;
}

// Add JavaScript Frameworks
JHtml::_('bootstrap.framework');

// Add template js
JHtml::_('script', 'template.js', array('version' => 'auto', 'relative' => true));

// Add my js script
JHtml::_('script', 'script.js', array('version' => 'auto', 'relative' => true));

// Add html5 shiv
JHtml::_('script', 'jui/html5.js', array('version' => 'auto', 'relative' => true, 'conditional' => 'lt IE 9'));

// Add Stylesheets
JHtml::_('stylesheet', 'template.css', array('version' => 'auto', 'relative' => true));

// Add Style.css
JHtml::_('stylesheet', 'style.css', array('version' => 'auto', 'relative' => true));

// Add Style.css
JHtml::_('stylesheet', 'override.css', array('version' => 'auto', 'relative' => true));

// Use of Google Font
if ($this->params->get('googleFont'))
{
	JHtml::_('stylesheet', 'https://fonts.googleapis.com/css?family=' . $this->params->get('googleFontName'));
	$this->addStyleDeclaration("
	h1, h2, h3, h4, h5, h6, .site-title {
		font-family: '" . str_replace('+', ' ', $this->params->get('googleFontName')) . "', sans-serif;
	}");
}

// Template color
if ($this->params->get('templateColor'))
{
	$this->addStyleDeclaration('
	body.site {
		border-top: 3px solid ' . $this->params->get('templateColor') . ';
		background-color: ' . $this->params->get('templateBackgroundColor') . ';
	}
	a {
		color: ' . $this->params->get('templateColor') . ';
	}
	.nav-list > .active > a,
	.nav-list > .active > a:hover,
	.dropdown-menu li > a:hover,
	.dropdown-menu .active > a,
	.dropdown-menu .active > a:hover,
	.nav-pills > .active > a,
	.nav-pills > .active > a:hover,
	.btn-primary {
		background: ' . $this->params->get('templateColor') . ';
	}');
}

// Check for a custom CSS file
JHtml::_('stylesheet', 'user.css', array('version' => 'auto', 'relative' => true));

// Check for a custom js file
JHtml::_('script', 'user.js', array('version' => 'auto', 'relative' => true));

// Load optional RTL Bootstrap CSS
JHtml::_('bootstrap.loadCss', false, $this->direction);

// Adjusting content width

if( ($left=$this->countModules('left')) xor ($right=$this->countModules('right')))
{
	$span = 'span9';
}
elseif($this->countModules('left') && $this->countModules('right'))
{
	$span = 'span6';
	$left = TRUE;
	$right = TRUE;
}
else
{
	$span = 'span12';
}

// Logo file or site title param
if ($this->params->get('logoFile'))
{
	$logo = '<img src="' . JUri::root() . $this->params->get('logoFile') . '" alt="' . $sitename . '" />';
}
elseif ($this->params->get('sitetitle'))
{
	$logo = '<span class="site-title" title="' . $sitename . '">' . htmlspecialchars($this->params->get('sitetitle'), ENT_COMPAT, 'UTF-8') . '</span>';
}
else
{
	$logo = '<span class="site-title" title="' . $sitename . '">' . $sitename . '</span>';
}
?>
<!DOCTYPE html>
<html lang="<?php echo $this->language; ?>" dir="<?php echo $this->direction; ?>">
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
	<meta name="viewport" content="width=device-width, initial-scale=1.0" />
	<jdoc:include type="head" />
</head>

<?php
	if($itemid == 101){
		$home = 'home';
	} else{
		$home = 'internal';
	}
?>

<body class="site <?php echo $home ?>">
	<!-- Body -->
	<div id="site" class="body">
		<div class="container<?php echo ($params->get('fluidContainer') ? '-fluid' : ''); ?>">
			
			<?php if ($this->countModules('topo')) : ?>
				<div class="topo" role="navigation">
					<div class="nav-collapse">
						<jdoc:include type="modules" name="topo" style="xhtml" />
					</div>
				</div>
			<?php endif; ?>


			<?php if ($this->countModules('topo2')) : ?>
				<div class="topo2" role="navigation">
					<div class="nav-collapse">
						<jdoc:include type="modules" name="topo2" style="xhtml" />
					</div>
				</div>
			<?php endif; ?>

			<?php if ($this->countModules('postopo')) : ?>
				<div class="postopo" role="navigation">
					<div class="nav-collapse">
						<jdoc:include type="modules" name="postopo" style="xhtml" />
					</div>
				</div>
			<?php endif; ?>

			<jdoc:include type="modules" name="banner" style="xhtml" />
			<div class="conteudo" id="conteudo">
				<?php if ($left) : ?>
					<!-- Begin Sidebar -->
					<div id="sidebar" class="span3">
						<div class="sidebar-nav">
							<jdoc:include type="modules" name="left" style="xhtml" />
						</div>
					</div>
					<!-- End Sidebar -->
				<?php endif; ?>
				<main id="content" role="main" class="<?php echo $span; ?>">
					<!-- Begin Content -->
					<jdoc:include type="message" />
					<jdoc:include type="component" />	
					<jdoc:include type="modules" name="conteudo" style="xhtml" />
					<!-- End Content -->
				</main>
				<?php if ($right) : ?>
					<div id="aside" class="span3">
						<!-- Begin Right Sidebar -->
						<jdoc:include type="modules" name="right" style="xhtml" />
						<!-- End Right Sidebar -->
					</div>
				<?php endif; ?>
			</div>
			<?php if ($this->countModules('conteudo2')) : ?>
				<div class="conteudo2" id="conteudo2">
					<div class="nav-collapse">
						<jdoc:include type="modules" name="conteudo2" style="xhtml" />
					</div>
				</div>
			<?php endif; ?>
			<?php if ($this->countModules('conteudo3')) : ?>
				<div class="conteudo3" id="conteudo3">
					<div class="nav-collapse">
						<jdoc:include type="modules" name="conteudo3" style="xhtml" />
					</div>
				</div>
			<?php endif; ?>
			<?php if ($this->countModules('conteudo4')) : ?>
				<div class="conteudo4" id="conteudo4">
					<div class="nav-collapse">
						<jdoc:include type="modules" name="conteudo4" style="xhtml" />
					</div>
				</div>
			<?php endif; ?>

			<?php if ($this->countModules('conteudo5')) : ?>
				<div class="conteudo5" id="conteudo5">
					<div class="nav-collapse">
						<jdoc:include type="modules" name="conteudo5" style="xhtml" />
					</div>
				</div>
			<?php endif; ?>

			<?php if ($this->countModules('conteudo6')) : ?>
				<div class="conteudo6" id="conteudo6">
					<div class="nav-collapse">
						<jdoc:include type="modules" name="conteudo6" style="xhtml" />
					</div>
				</div>
			<?php endif; ?>

			<?php if ($this->countModules('conteudo7')) : ?>
				<div class="conteudo7" id="conteudo7">
					<div class="nav-collapse">
						<jdoc:include type="modules" name="conteudo7" style="xhtml" />
					</div>
				</div>
			<?php endif; ?>
			<!-- Footer -->
			<footer class="footer" role="contentinfo">
					<?php if ($this->countModules('footer')) : ?>
						<div class="footer">
							<div class="nav-collapse">
								<jdoc:include type="modules" name="footer" style="xhtml" />
							</div>
						</div>
					<?php endif; ?>
			</footer>
		</div>
	</div>
	<jdoc:include type="modules" name="debug" style="none" />
</body>
</html>